#include <utility.h>
#include <userint.h>

/*****************************************************************************/
/*   Copyright 1998 National Instruments Corporation.  All Rights Reserved.  */
/*****************************************************************************/


/*=============== R3267/73 Spectrum Analyzer OPT66(BLUETOOTH)================*/
/* LabWindows/CVI 5.0 Instrument Driver										 */
/* Original Release: Novenber/29/2000										 */
/* By: Ai Onishi,    Supervisor: Yohei Hirakoso								 */
/*                   SPSpj Gyoda R&D,  ADVANTEST CORPORATION				 */
/*                   Home Page; http://www.advantest.co.jp					 */
/*                   e-mail;    info@advantest.co.jp						 */
/*---------------------------------------------------------------------------*/
/* Modification History: 													 */
/*         Version 1.0 :  Novenber/29/2000									 */
/*																			 */
/*===========================================================================*/
 


#include <visa.h>
#include <ansi_c.h>
#include <formatio.h>
#include "avopt66.h"

#define avopt66_REVISION    "Rev 1.0, 09/00, CVI 5.0.1" /*  Instrument driver revision */
#define BUFFER_SIZE         1024L         				/* File I/O buffer size */
    

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/


/****
 ****	Opt66 Static Command Arr
 ****/

/* BLUETOOTH Standard */
static ViString modSysBtArr[]={"USA","EUROPE","SPAIN","FRANCE","JAPAN"};
static ViString measMdArr[]={"BURST","","CONT"};
static ViString syncTypArr[]={"NO","LAP"};
static ViString mfltMdArr[]={"NARW","WIDE"};      
static ViString freqInptArr[]={"CHL","FREQ"};
static ViString inputArr[]={"RF","IQ"};
static ViString bbInputArr[]={"AC","DC"};
static ViString iqModeArr[]={"NORM","INV"};


/* arr for comon function */
static ViString switchArr[] = {"OFF","ON"};
static ViString upDownArr[]={"DOWN","UP"};

/* Edit Channel */
static ViString edtChArr[]={"UP1","DN1","UP2","DN2","UP3","DN3"};

/* Channel Enable */
static ViString enableArr[]={"DSBL","ENBL"};

/* Y Scale Sel */
static ViString YscaleModeArr[] ={"TDP","OOR","FDP"};
static ViString YscaleDivArr[] ={"P10DB","P5DB","P2DB"};

/* Trigger */
static ViString trgWFSrcArr[] = {"TRGSRC FREE","TRGSRC VIDEO","TRGSRC IF","TRGSRC EXT"};
static ViString trgSlopeArr[] = {"TRGSLP FALL","TRGSLP RISE"};

/* parameter Select */
static ViString DetArr[]={"DET NRM","DET POS","DET NEG","DET SMP"};
static ViString LvlUnitArr[] ={"UNIT DBM", "UNIT W", "UNIT DBUV"};       

/* Table */
static ViString selTblArr[] = {"TBL 1","TBL 2","TBL 3"};
static ViString tableFArr[] = {"TBLF ABS","TBLF REL"};

/* Modulation BT */
static ViString modTrgArr[]={"FREE","IF","EXT"};
static ViString slopeArr[]={"FALL","RISE"};
static ViString fmBitSqArr[]={"STD","RND"};

/* Mod Param */
static ViString lkupFRanArr[]={"1M","100K"};
static ViString lkupYSclArr[]={"P2K","P10K","P20K","P100K","P200K"};

/* Avarage */
static ViString avgFunctionArr[] = {"TDPAVG", "OORAVG", "TDSAVG", "FDPAVG", "OBWAVG",
									"DTSAVG", "DTMAVG", "SPRAVG","FDSAVG","FMAVG",
									"TXAVG","LKUPAVG"}; 
									
/* Mesurement Start */
static ViString funcMeasureArr[] = {"TDPMEAS","OORMEAS","TDSMEAS","FDPMEAS", "OBWMEAS", "DTSMEAS",
									"DTMMEAS","SPRMEAS","FDSMEAS","FMDEV","TXPWR","LKUPTM","SI"};
									
/* Template Edit */
static ViString UpLowArr[]={"LOW","UP"};

/* Preselecter */
static ViString preselArr[]={"36G","16G"};

/* Graph Type */
static ViString gTypFMArr[] = {"FMGTYP BIT","FMGTYP EYE","FMGTYP DEMOD","FMGTYP SPECTRUM","FMGTYP FREQDRIFT"};


/* Auto Level */
static ViString autoLvlFunctionArr[] = {"TDPAUTOLVL","OORAUTOLVL","TDSAUTOLVL",
										"FDPAUTOLVL","OBWAUTOLVL","DTSAUTOLVL",
										"DTMAUTOLVL","SPRAUTOLVL","FDSAUTOLVL",
										"AUTOLVL"};

/* Gate */
static ViString gateExSTDArr[] = {"DTMSETSTD","FDPSETSTD"};
static ViString trgFdpSrcArr[] ={"FREE","VIDEO","IF","EXT"};

/*TDS Param*/
static ViString resultArr[] = {"RES RMS","RES PK"};

/* DTS Param */
static ViString FreqSetArr[] ={"FRMD CFSP","FRMD STSP"};

static ViString dtsResultArr[] = {"RES MKR", "RES REL","RES ABS"};
static ViString refMkrArr[] = {"REF MOD","REF MKR"};

/* FM Deviation */
static ViString FreqDevGraphArr[] = {"BIT","EYE","DEMOD","SPECTRUM","FREQDRIFT","FREQDRIFT"};
static ViString FMDevGqueArr[]={"GPHX?","GPHX?","DEMOD?","SGPHX?","FGPHX1?","FGPHX2?"};
static ViString FMDevGqueYArr[]={"GPHY?","GPHY?"," ","SGPHY?","FGPHY1?","FGPHY2?"};

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* avopt66_stringValPair is used in the avopt66_errorMessage function        */
/* avopt66_statusDataRanges is used to track session dependent status & ranges*/
/*===========================================================================*/
typedef struct  avopt66_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  avopt66_tStringValPair;
 
struct avopt66_statusDataRanges {
    ViInt16 traceRange;
    ViInt16 harmonics;
    ViChar instrDriverRevision[256];
};

typedef struct avopt66_statusDataRanges *avopt66_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean avopt66_invalidViBooleanRange (ViBoolean val);
ViBoolean avopt66_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max);
ViBoolean avopt66_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);
ViBoolean avopt66_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max);
ViBoolean avopt66_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max);
ViBoolean avopt66_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max);
ViBoolean avopt66_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max);
ViBoolean avopt66_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViStatus avopt66_waitOnVisaEvent (ViSession instrSession, ViEventType eventTypeIn,
	ViUInt32 tmoIn, ViPUInt16 STB);
ViStatus avopt66_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
	ViStatus currentStatus);
ViStatus avopt66_defaultInstrSetup (ViSession openInstrSession);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Standard Setting BLUETOOTH                                      */
/* Purpose:  This function sets the communication type and link of the signal*/
/*           under TRANSIENT measurement.									 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_parameter_stdBT (ViSession instrumentHandle,
                                          ViInt16 CNAssignment, ViBoolean link,
                                          ViBoolean hoppingCatch,
                                          ViInt16 measMode, ViInt16 burstLength,
                                          ViInt16 searchLength,
                                          ViBoolean syncType,
                                          ViBoolean fillterMode, ViInt32 lap,
                                          ViReal64 offsetLevel,
                                          ViBoolean frequencyInput,
                                          ViBoolean input,
                                          ViBoolean basebandInput,
                                          ViBoolean IQInverse,
                                          ViBoolean contAutoLevelSet)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE],SyncBuf[20];
    ViStatus avopt66_status = VI_SUCCESS;
    
	if (avopt66_invalidViInt16Range (CNAssignment, 0, 4))
        return VI_ERROR_PARAMETER2;
	if (avopt66_invalidViBooleanRange (link))
        return VI_ERROR_PARAMETER3;
    if (avopt66_invalidViBooleanRange (hoppingCatch))
        return VI_ERROR_PARAMETER4;    
	if ((avopt66_invalidViInt16Range (measMode, 0, 2)) || (measMode == 1))      
        	return VI_ERROR_PARAMETER5;    
    if (avopt66_invalidViInt16Range (burstLength, 68, 3000))
        return VI_ERROR_PARAMETER6;    
    if (avopt66_invalidViInt16Range (searchLength, 3, 163))
        return VI_ERROR_PARAMETER7;    
	if (avopt66_invalidViBooleanRange (syncType))
        return VI_ERROR_PARAMETER8;
    if (avopt66_invalidViBooleanRange (fillterMode))
        return VI_ERROR_INSTR_PARAMETER9;         
    
    if (avopt66_invalidViReal64Range (offsetLevel, -100.0, 100.0))
        return VI_ERROR_INSTR_PARAMETER11;
    if (avopt66_invalidViBooleanRange (frequencyInput))
        return VI_ERROR_INSTR_PARAMETER12;
    if (avopt66_invalidViBooleanRange (input))
        return VI_ERROR_INSTR_PARAMETER13;
    if (avopt66_invalidViBooleanRange (basebandInput))
        return VI_ERROR_INSTR_PARAMETER14;
    if (avopt66_invalidViBooleanRange (IQInverse))
        return VI_ERROR_INSTR_PARAMETER15;
    if (avopt66_invalidViBooleanRange (contAutoLevelSet))
        return VI_ERROR_INSTR_PARAMETER16;        

	Fmt (wrtBuf, "%s<CHNOAS %s; LINK %s; HOPCATCH %s; MEASMD %s;BRSTLEN %d; SRCHLEN %d; SYNC %s;LAP %x", 
		 modSysBtArr[CNAssignment],
		 upDownArr[link],
		 switchArr[hoppingCatch],
		 measMdArr[measMode],
		 burstLength,
		 searchLength,
		 syncTypArr[syncType],
		 lap);
    
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt66_status;
	
	Fmt (wrtBuf, "%s<MFLTMD %s; RO %fDB; FINPMD %s;INPUT %s; BBINPUT %s; IQMD %s; ALS %s", 
		 mfltMdArr[fillterMode],
		 offsetLevel,
		 freqInptArr[frequencyInput],
		 inputArr[input],
		 bbInputArr[basebandInput],
		 iqModeArr[IQInverse],
		 switchArr[contAutoLevelSet]);
		 
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt66_status;
		 
    return avopt66_status;       
}

/*===========================================================================*/
/* Function: Channel Setting                                            	 */
/* Purpose: This function sets the Channel Number.                           */
/*===========================================================================*/

ViStatus CVIFUNC avopt66_number_setChannel (ViSession instrumentHandle,
                                            ViInt16 channelNo)
{
    ViUInt32 retCnt = 0;

    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;
 
    if (avopt66_invalidViInt16Range(channelNo, -12800, 12800))
        return VI_ERROR_PARAMETER2; 
        
    Fmt (wrtBuf, "%s<CH %d",channelNo);
        
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;      
}



/*===========================================================================*/
/* Function: Channel Edit 		                                             */
/* Purpose: This function edits channel setting.                             */
/*===========================================================================*/

ViStatus CVIFUNC avopt66_ChEdt (ViSession instrumentHandle, ViBoolean settoSTD,
                                ViInt16 editChannel, ViInt16 startCHNum,
                                ViInt16 stopCHNum, ViReal64 baseFreqHz,
                                ViReal64 channelSpaceHz, ViInt16 CHOffset)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if (avopt66_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;
    if (avopt66_invalidViInt16Range (editChannel, 0, 5))
        return VI_ERROR_PARAMETER3;
    if (avopt66_invalidViInt16Range (startCHNum, -12800, 12800))
        return VI_ERROR_PARAMETER4;
    if (avopt66_invalidViInt16Range (stopCHNum, -12800, 12800))
        return VI_ERROR_PARAMETER5;
    if (avopt66_invalidViReal64Range (baseFreqHz, 0.0, 100000000000.0))
        return VI_ERROR_PARAMETER6;
    if (avopt66_invalidViReal64Range (channelSpaceHz, -100000000.0, 100000000.0))
        return VI_ERROR_PARAMETER7;
    if (avopt66_invalidViInt16Range (CHOffset, -12800, 12800))
        return VI_ERROR_PARAMETER8;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<CHSETSTD");
    	total += NumFmtdBytes();
	}
	else
	{
		Fmt (wrtBuf, "%s<CHED%s %d,%d,%fHZ,%fHZ,%d", edtChArr[editChannel],
    	startCHNum,stopCHNum,baseFreqHz,channelSpaceHz,CHOffset);
    	total += NumFmtdBytes();
	}

    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;  
   
}


/*===========================================================================*/
/* Function: Channel Enable ON/OFF                                           */
/* Purpose:  This function sets the state of Channel table enable.           */
/*                                      									 */
/*===========================================================================*/

ViStatus CVIFUNC avopt66_ch_Enable (ViSession instrumentHandle,
                                    ViBoolean channel1, ViBoolean channel2,
                                    ViBoolean channel3)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if (avopt66_invalidViBooleanRange (channel1))
        return VI_ERROR_PARAMETER2;	
    if (avopt66_invalidViBooleanRange (channel2))
        return VI_ERROR_PARAMETER3;	   
    if (avopt66_invalidViBooleanRange (channel3))
        return VI_ERROR_PARAMETER4;	           

	Fmt (wrtBuf, "%s<CHTBL1 %s;CHTBL2 %s;CHTBL3 %s",
		enableArr[channel1],
		enableArr[channel2],
		enableArr[channel3]);
    total += NumFmtdBytes();


    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;      
}
/*===========================================================================*/
/* Function: Y Scale Setup			                                         */
/* Purpose:  This function sets the state of Channel table enable.           */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_Yscale_setup (ViSession instrumentHandle,
                                       ViInt16 measMode, ViInt16 division)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if (avopt66_invalidViInt16Range (measMode, 0, 2))
        return VI_ERROR_PARAMETER2;     
    if (avopt66_invalidViInt16Range (division, 0, 2))
        return VI_ERROR_PARAMETER3;
        
	Fmt (wrtBuf, "%s<%sDIV %s",
		YscaleModeArr[measMode],
		YscaleDivArr[division]);
    total += NumFmtdBytes();


    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;      
}    


/*===========================================================================*/
/* Function: TDP Trigger Conditions                                          */
/* Purpose:  This function is used for time domain Power. This               */
/*           function sets the trigger parameters.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_confTrgConTDP (ViSession instrumentHandle,
                                        ViInt16 triggerSource,
                                        ViBoolean trgSlope, 
                                        ViInt16 triggerLevel,
                                        ViInt16 triggerPosition,
                                        ViReal64 delayTimes)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if (avopt66_invalidViInt16Range (triggerSource, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avopt66_invalidViBooleanRange (trgSlope))
        return VI_ERROR_PARAMETER3;
    if (avopt66_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER4;
    if (avopt66_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER5;
    if (avopt66_invalidViReal64Range (delayTimes, -1000.0, 1000.0))
        return VI_ERROR_PARAMETER6;

	Fmt (wrtBuf, "%s<%s", trgWFSrcArr[triggerSource]);
	total += NumFmtdBytes();
	 
	if (triggerSource > 0 ){
		
		Fmt (wrtBuf, "%s[a]<; %s", trgSlopeArr[trgSlope]);
		total += NumFmtdBytes();

		if (triggerSource < 3 ){
			Fmt (wrtBuf, "%s[a]<; TRGLVL %dENT", triggerLevel);
			total += NumFmtdBytes();
		}
		
		Fmt (wrtBuf, "%s[a]<; TRGPOS %dENT; TRGDT %fSC", triggerPosition, delayTimes);
    	total += NumFmtdBytes(); 
	}
    
    
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;       
    
}

/*===========================================================================*/
/* Function: TDP Window Setup					                             */
/* Purpose:  This function is used for time domain Power.					 */
/*           This function sets the Window setup parameters.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_windowSetTDP (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViBoolean window,
                                       ViReal64 winPosS, ViReal64 winWidS)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;    

    if (avopt66_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;
    if (avopt66_invalidViBooleanRange (window))
        return VI_ERROR_PARAMETER3;
    if (avopt66_invalidViReal64Range (winPosS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER4;   
    if (avopt66_invalidViReal64Range (winWidS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER5; 

    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<TDPSETSTD");
    	total += NumFmtdBytes();
    }
    else
    {
        Fmt (wrtBuf, "%s<TDPWDO %s; TDPWPOS %fSC; TDPWWID %fSC",
        	switchArr[window],
    		winPosS, winWidS);
    	total += NumFmtdBytes();
    }
               
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;    
}


/*===========================================================================*/
/* Function: TDP Template Setup					                             */
/* Purpose:  This function is used for time domain Power.					 */
/*           This function sets the Template setup parameters.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_template_SetTDP (ViSession instrumentHandle,
                                          ViBoolean template, ViReal64 shiftXS,
                                          ViReal64 shiftYdB)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;    

	if (avopt66_invalidViBooleanRange (template))
        return VI_ERROR_PARAMETER2;

    if (avopt66_invalidViReal64Range (shiftXS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER3;   
    if (avopt66_invalidViReal64Range (shiftYdB, -200.0, 100.0))
        return VI_ERROR_PARAMETER4; 

    Fmt (wrtBuf, "%s<TDPTMPL %s; TDPTMPLSX %fSC; TDPTMPLSY %fDB", 
    	switchArr[template],
    	shiftXS, 
    	shiftYdB);
    
    total += NumFmtdBytes();
    
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;      
}


/*===========================================================================*/
/* Function: TDP Template Edit					                             */
/* Purpose:  This function is used for Time domain Power.					 */
/*           This function sets the Table data of Template.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_template_editTDP (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean templateSelect,
                                           ViReal64 templateTimes,
                                           ViReal64 templateLevel)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;    

	if (avopt66_invalidViBooleanRange (templateSelect))
        return VI_ERROR_PARAMETER3;
	if (avopt66_invalidViInt16Range (operation,0,2 ))
        return VI_ERROR_PARAMETER2;
 
 
    Fmt (wrtBuf, "%s<TDPTMPLSEL %s", UpLowArr[templateSelect]);
	total += NumFmtdBytes();



	switch (operation)
	{
	case 0: /* copy from STD */
		Fmt (wrtBuf, "%s[a]<; TDPTMPLCP");
		break;
		
	case 1: /* entry */
		if (avopt66_invalidViReal64Range (templateTimes, -1.0, 1.0))
        	return VI_ERROR_PARAMETER4;
/*		if (avopt66_invalidViReal64Range (templateLevel, -200.0, 100.0))
        	return VI_ERROR_PARAMETER5; 
*/
		Fmt (wrtBuf, "%s[a]<; TDPTMPLED %fSC,%fDB", templateTimes, templateLevel);
		break;
	case 2: /* Table init */
		 Fmt(wrtBuf, "%s[a]<; TDPTMPLCLR"); 
		break;
	}
		total += NumFmtdBytes(); 
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;    
}

/*===========================================================================*/
/* Function: TDP Parameter Setup				                             */
/* Purpose:  This function is used for Time domain Power.					 */
/*           This function sets the parameters for Measurement.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_parameter_setTDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean TMPLCoupleofPWR,
                                           ViReal64 templateLimit,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;    

    if (avopt66_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;


    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<TDPSETSTD");
    }
    else
    {
    	if (avopt66_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt66_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
    	if (avopt66_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_PARAMETER5;
   		if (avopt66_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER6;        

        Fmt (wrtBuf, "%s<TDP%s; TDP%s; TDPTMPLPW %s; TDPTMPLBTM %fDB; TDPJDG %s; TDPJDGUP %fDB; TDPJDGLOW %fDB",
        	DetArr[detector], LvlUnitArr[displayUnit], switchArr[TMPLCoupleofPWR],
        	templateLimit, switchArr[judgement], upperLimit,  lowerLimit);

    }
    	total += NumFmtdBytes();                 
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;        
}


/*===========================================================================*/
/* Function: On/Off Raito Trigger   				                         */
/* Purpose:  This function is used for On/Off Raito (Time Domain).			 */
/*           This function sets the parameters for Trigger. 				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_trigger_setOOR (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if (avopt66_invalidViInt16Range (triggerSource, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (avopt66_invalidViBooleanRange (trgSlope))
        return VI_ERROR_PARAMETER3;
    if (avopt66_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER4;
    if (avopt66_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER5;
    if (avopt66_invalidViReal64Range (delayTimes, -1000.0, 1000.0))
        return VI_ERROR_PARAMETER6;

	Fmt (wrtBuf, "%s<OOR%s", trgWFSrcArr[triggerSource]);
	total += NumFmtdBytes();
	 
	if (triggerSource > 0 ){
		
		Fmt (wrtBuf, "%s[a]<; OOR%s", trgSlopeArr[trgSlope]);
		total += NumFmtdBytes();

		if (triggerSource < 3 ){
			Fmt (wrtBuf, "%s[a]<; OORTRGLVL %dENT", triggerLevel);
			total += NumFmtdBytes();
		}
		
		Fmt (wrtBuf, "%s[a]<;OORTRGPOS %dENT; OORTRGDT %fSC", triggerPosition, delayTimes);
    	total += NumFmtdBytes(); 
	}
	
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;       
    	    
}

/*===========================================================================*/
/* Function: On/Off Raito Window						                     */
/* Purpose:  This function is used for time domain wave form analyzing. This */
/*           function sets the trigger parameters.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_window_setOOR (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 onPositionS, ViReal64 onWidthS,
                                        ViReal64 offPositionS,
                                        ViReal64 offWidthS)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;    

    if (avopt66_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;
    if (avopt66_invalidViBooleanRange (window))
        return VI_ERROR_PARAMETER3;
    if (avopt66_invalidViReal64Range (onPositionS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER4;   
    if (avopt66_invalidViReal64Range (onWidthS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER5; 
    if (avopt66_invalidViReal64Range (offPositionS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER6;   
    if (avopt66_invalidViReal64Range (offWidthS, 0.0, 1000.0))
        return VI_ERROR_PARAMETER7; 
        
    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<OORSETSTD");
    }
    else
    {
        Fmt (wrtBuf, "%s<OORWDO %s; OORWONPOS %fSC; OORWONWID %fSC; OORWOFPOS %fSC; OORWOFWID %fSC",
        	switchArr[window], onPositionS, onWidthS, offPositionS, offWidthS);
    }
    	total += NumFmtdBytes();           
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;    	    
}

/*===========================================================================*/
/* Function: On/Off Raito Parameter Setup				                     */
/* Purpose:  This function is used for Time domain Power.					 */
/*           This function sets the parameters for Measurement.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_param_setOOR (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViInt16 displayUnit, ViBoolean judgement,
                                       ViReal64 upperLimit)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;    

    if (avopt66_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;


    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<OORSETSTD");
    	total += NumFmtdBytes();
    }
    else
    {
    	if (avopt66_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt66_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
   		if (avopt66_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER5;        
    
    
        Fmt (wrtBuf, "%s<OOR%s; OOR%s; OORJDG %s; OORJDGUP %fDB",
        	DetArr[detector], LvlUnitArr[displayUnit], switchArr[judgement], upperLimit);
    	total += NumFmtdBytes();
    }
               
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;    
}
/*===========================================================================*/
/* Function: TDS Trigger Set	                                             */
/* Purpose:  This function is used for time domain Spurious. This            */
/*           function sets the trigger parameters.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_trigger_setTDS (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if (avopt66_invalidViInt16Range (triggerSource, 0, 3))
        return VI_ERROR_PARAMETER2;
    else if (triggerSource == 1)			/* Video Mode none */
    	return VI_ERROR_PARAMETER2;
    	
    if (avopt66_invalidViBooleanRange (trgSlope))
        return VI_ERROR_PARAMETER3;
    if (avopt66_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER4;
    if (avopt66_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER5;    
    if (avopt66_invalidViReal64Range (delayTimes, -1000.0, 1000.0))
        return VI_ERROR_PARAMETER6;

    Fmt (wrtBuf, "%s<TDS%s;TDS%s", trgWFSrcArr[triggerSource],
    	trgSlopeArr[trgSlope]);
    total += NumFmtdBytes();
    
    
	if (triggerSource > 1 ) {
		
		if (triggerSource == 2) {
	    	Fmt (wrtBuf, "%s[a]<;TDSTRGLVL %dENT", triggerLevel);
	    	total += NumFmtdBytes();		
		}
		
	    Fmt (wrtBuf, "%s[a]<;TDSTRGPOS %dENT;TDSTRGDT %fSC", triggerPosition, delayTimes);
	    total += NumFmtdBytes();     
	}

    
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;       
}

/*===========================================================================*/
/* Function: TDS Table Edit						                     		 */
/* Purpose:  This function is used for Time domain Spurious.				 */
/*           This function sets the Table for Measurement.					 */
/*===========================================================================*/

ViStatus CVIFUNC avopt66_table_editTDS (ViSession instrumentHandle,
                                        ViInt16 operation, ViInt16 tableNumber,
                                        ViBoolean freqType, ViReal64 carrierFreq,
                                        ViReal64 frequency, ViReal64 level)
{
    ViUInt32 i,retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;
    
	if (avopt66_invalidViInt16Range (operation, 0, 5))
        return VI_ERROR_PARAMETER2;    
	if (avopt66_invalidViInt16Range (tableNumber, 1, 3))
        return VI_ERROR_PARAMETER3;

	switch (operation)
	{
	case 0: /* LOAD TABLE */
		Fmt (wrtBuf, "%s<RCL%s", selTblArr[tableNumber-1]);
    	total += NumFmtdBytes();
		break;
	case 1: /* SAVE TABLE */
		Fmt (wrtBuf, "%s<SVS%s", selTblArr[tableNumber-1]);
    	total += NumFmtdBytes();
		break;
	case 2: /* SELECT TABLE ONLY */
		Fmt (wrtBuf, "%s<TDS%s", selTblArr[tableNumber-1]);
    	total += NumFmtdBytes();
		break;
	case 3: /* DATA ENTRY */
		Fmt (wrtBuf, "%s<TDSTBLED %fHZ,%fDB",frequency, level);
    	total += NumFmtdBytes();
		break;
	case 4: /* INIT TABLE */
		Fmt (wrtBuf, "%s<TDSCLR");
    	total += NumFmtdBytes();
		break;
	case 5: /* TABLE FREQ & CARRER FREQ */
		if (avopt66_invalidViBooleanRange (freqType))
        	return VI_ERROR_PARAMETER4;
        if (avopt66_invalidViReal64Range (carrierFreq, 0.0, 8000000000.0))
        	return VI_ERROR_PARAMETER5;	
		Fmt (wrtBuf, "%s<TDS%s; CF %fHZ",tableFArr[freqType],carrierFreq);
		break;
	}
	total += NumFmtdBytes();
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        	return avopt66_status;    
    return avopt66_status;     
}

/*===========================================================================*/
/* Function: TDS Parameter Setup				                     		 */
/* Purpose:  This function is used for Time domain Spurious.				 */
/*           This function sets the parameters for Measurement.				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_param_setTDS (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViInt16 displayUnit, ViBoolean judgement,
                                       ViBoolean result, ViReal64 peakMKRYDelta,
                                       ViReal64 multiplier,
                                       ViBoolean preselector)
{
	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;    

    if (avopt66_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

    if ( settoSTD == VI_ON )
    {
        Fmt (wrtBuf, "%s<TDSSETSTD");
    	total += NumFmtdBytes();
    }
    else
    {
    	if (avopt66_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt66_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
   		if (avopt66_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER5;        
	    if (avopt66_invalidViBooleanRange (result))
        	return VI_ERROR_PARAMETER6;    
   		if (avopt66_invalidViReal64Range (peakMKRYDelta, 0.1, 10.0))
        	return VI_ERROR_PARAMETER7;
        if (avopt66_invalidViReal64Range (multiplier, 1.0, 100.0))
        	return VI_ERROR_PARAMETER8;
        if (avopt66_invalidViBooleanRange (preselector))
        	return VI_ERROR_INSTR_PARAMETER9; 	
         
        Fmt (wrtBuf, "%s<TDS%s; TDS%s; TDSJDG %s; TDS%s; TDSPKMKY %fENT; TDSMULTI %fENT;TDSPRE %s",
        	DetArr[detector], 
        	LvlUnitArr[displayUnit],
        	switchArr[judgement], 
        	resultArr[(ViInt16)result],
        	peakMKRYDelta, multiplier,preselArr[preselector]);

    	total += NumFmtdBytes();
    }
              
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;    	    
}

/*===========================================================================*/
/* Function: Gated Spectrum	Trigger 			                             */
/* Purpose:  This function is used for Freq domain wave form analyzing. This */
/*           function sets the Gate Setup Trigger parameters.	     		 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_gateTrg_setFDP (ViSession instrumentHandle,
                                         ViBoolean gateSource,
                                         ViBoolean settoSTD,
                                         ViBoolean measurementMode,
                                         ViReal64 gatePositions,
                                         ViReal64 gateWidths,
                                         ViInt16 trgSource,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViBoolean gatedSweep,
                                         ViInt16 detector)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if (avopt66_invalidViBooleanRange (gateSource))
        return VI_ERROR_PARAMETER2;
    if (avopt66_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER3;
    if (avopt66_invalidViBooleanRange (measurementMode))
        return VI_ERROR_PARAMETER4;
    if (avopt66_invalidViReal64Range (gatePositions, 0, 1))
        return VI_ERROR_PARAMETER5;
    if (avopt66_invalidViReal64Range (gateWidths, 0, 1))
        return VI_ERROR_PARAMETER6;
	if (avopt66_invalidViInt16Range (trgSource,0,3))
        return VI_ERROR_PARAMETER7;
    if (avopt66_invalidViInt16Range (triggerPosition, 0, 100))
        return VI_ERROR_PARAMETER8;
    if (avopt66_invalidViReal64Range (delayTimes, -1000.0, 1000.0))
        return VI_ERROR_INSTR_PARAMETER9;
    if (avopt66_invalidViBooleanRange (trgSlope))
        return VI_ERROR_INSTR_PARAMETER10;
    if (avopt66_invalidViInt16Range (triggerLevel, 0, 100))
        return VI_ERROR_INSTR_PARAMETER11;
    if (avopt66_invalidViBooleanRange (gatedSweep))
        return VI_ERROR_INSTR_PARAMETER12;    
    if (avopt66_invalidViInt16Range (detector, 0, 3))
        return VI_ERROR_INSTR_PARAMETER13;

   	Fmt (wrtBuf, "%s<TGTSRC TRG; TGTTRG %s; TGT%s; TGTTRGLVL %d; TGTTRGPOS %d; TGTTRGDT %f[e]SC",
    	 trgFdpSrcArr[trgSource],
    	 trgSlopeArr[trgSlope], 
    	 triggerLevel, triggerPosition, delayTimes);
    total += NumFmtdBytes();
	
   	if ( settoSTD == 1)          /* Set to STD: On */
    	Fmt (wrtBuf, "%s[a]<; TGTSETUP OFF; %s", gateExSTDArr[measurementMode]);
   		
   	else						 /* Set to STD: Off */
   		Fmt (wrtBuf, "%s[a]<; TGTPOS %f[e]SC; TGTWID %f[e]SC", gatePositions,gateWidths);
   	total += NumFmtdBytes();
   	
	if (gateSource == 0)      /* Gate Source: EXT */
	{
		Fmt (wrtBuf, "%s[a]<; TGTSRC EXT");
		total += NumFmtdBytes();
	}	

   	Fmt (wrtBuf, "%s[a]<; TGTSWP %s; TGT%s; TGTSETUP OFF", switchArr[gatedSweep],DetArr[detector]);
    total += NumFmtdBytes();
    
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;    
}

/*===========================================================================*/
/* Function: FDP Window Set						                             */
/* Purpose:  This function is used for Freq domain wave form analyzing. This */
/*           function sets the Gate Sweep Window.		     				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_window_setFDP (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 positionHz, ViReal64 widthHz)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;
    
    if (avopt66_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<FDPSETSTD");	
	}
	else
	{
	    if (avopt66_invalidViBooleanRange (window))
        	return VI_ERROR_PARAMETER3;
	    if (avopt66_invalidViReal64Range (positionHz, 0.0, 8.0e+9))
        	return VI_ERROR_PARAMETER4;
    	if (avopt66_invalidViReal64Range (widthHz, 0.0, 8.0e+9))
        	return VI_ERROR_PARAMETER5;
        	
		Fmt (wrtBuf, "%s<FDPWDO %s; FDPWPOS %fHZ; FDPWWID %fHZ", 
			switchArr[window], positionHz, widthHz);
	}
    
    total += NumFmtdBytes();
    
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;

}

/*===========================================================================*/
/* Function: FDP Parameter Set					                             */
/* Purpose:  This function is used for Freq domain wave form analyzing. This */
/*           function sets the Gate Sweep Window.		     				 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_parameter_setFDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean gatedSweep,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;
    
    if (avopt66_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<FDPSETSTD");	
	    total += NumFmtdBytes();
	}
	else
	{
    	if (avopt66_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt66_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
		if (avopt66_invalidViBooleanRange (gatedSweep))
        	return VI_ERROR_PARAMETER5;     	
   		if (avopt66_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER6;        

        Fmt (wrtBuf, "%s<FDP%s; FDP%s;TGTSWP %s; FDPJDG %s; FDPJDGUP %f; FDPJDGLOW %f",
        	DetArr[detector], LvlUnitArr[displayUnit],
        	switchArr[gatedSweep], switchArr[judgement],
        	upperLimit,  lowerLimit);
    	total += NumFmtdBytes();	
	}
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;	
}

/*===========================================================================*/
/* Function: OBW Parameter Set                                               */
/* Purpose:  This function sets the parameters for OBW measurement.			 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_parameter_setOBW (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViReal64 OBW,
                                           ViInt16 detector, ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;
    
    if (avopt66_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<OBWSETSTD");	
	    total += NumFmtdBytes();    
	
	}
	else
	{
	    if (avopt66_invalidViReal64Range (OBW, 0.5, 99.5))
        	return VI_ERROR_PARAMETER3;
    	if (avopt66_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER4;
   		if (avopt66_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER5;        
	    if (avopt66_invalidViReal64Range (upperLimit, 0, 100e+6))
        	return VI_ERROR_PARAMETER6;
	    if (avopt66_invalidViReal64Range (lowerLimit, 0, 100e+6))
        	return VI_ERROR_PARAMETER7;

        Fmt (wrtBuf, "%s<OBWPER %f; OBW%s; OBWJDG %s; OBWJDGUP %fHZ; OBWJDGLOW %fHZ",
        	OBW, DetArr[detector],
			switchArr[judgement], upperLimit, lowerLimit);
    	total += NumFmtdBytes();	
	}
        
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;		
}


/*===========================================================================*/
/* Function: DT Parameter Set                                                */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_parameter_setDT (ViSession instrumentHandle,
                                          ViBoolean settoSTD, ViInt16 detector,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViReal64 templateLimit,
                                          ViBoolean judgement,
                                          ViBoolean freqSetting, ViInt16 result,
                                          ViBoolean refPower,
                                          ViReal64 symbolRate1T,
                                          ViReal64 rolloffFactor)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;
    
    if (avopt66_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<DTSSETSTD");	
	    total += NumFmtdBytes();
	}
	else
	{
    	if (avopt66_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
   		if (avopt66_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER4;
   		if (avopt66_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_PARAMETER5; 
   		if (avopt66_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER7;  
   		if (avopt66_invalidViBooleanRange (freqSetting))
        	return VI_ERROR_PARAMETER8;         	
    	if (avopt66_invalidViInt16Range (result, 0, 2))
        	return VI_ERROR_INSTR_PARAMETER9;
   		if (avopt66_invalidViBooleanRange (refPower))
        	return VI_ERROR_INSTR_PARAMETER10;
        if (avopt66_invalidViReal64Range (symbolRate1T, 1, 100e+9))
        	return VI_ERROR_INSTR_PARAMETER11;
		if (avopt66_invalidViReal64Range (rolloffFactor, 0.01, 0.99))
        	return VI_ERROR_INSTR_PARAMETER12;

        Fmt (wrtBuf, "%s<DTS%s; DTS%s; DTS%s; DTS%s; DTS%s;DTSTMPLPW %s;DTSTMPLBTM %fDB; DTSJDG %s; DTSSYMRT %fHZ; DTSRFACT %f",
        	FreqSetArr[freqSetting], DetArr[detector], dtsResultArr[result],
        	refMkrArr[refPower], LvlUnitArr[displayUnit], switchArr[TMPLCoupleofPWR],
			templateLimit, switchArr[judgement], symbolRate1T, rolloffFactor);

    	total += NumFmtdBytes();	
	}
        
    
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;	    
}

/*===========================================================================*/
/* Function: DT Template Set                                                 */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_template_setDT (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz)
{
  	ViUInt32 i,retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;    

    if (avopt66_invalidViBooleanRange (template))
       	return VI_ERROR_PARAMETER2;
    if (avopt66_invalidViReal64Range (shiftXHz, -8.0e+9, 8.0e+9))
       	return VI_ERROR_PARAMETER3;   
    if (avopt66_invalidViReal64Range (shiftYdB, -200.0, 100))
       	return VI_ERROR_PARAMETER4; 
	if (avopt66_invalidViReal64Range (deltaXHz, -100e+6, 100e+6 ))
       	return VI_ERROR_PARAMETER5; 

    Fmt (wrtBuf, "%s<DTSTMPL %s; DTSTMPLSX %fHZ; DTSTMPLSY %fDB; DTSTMPLDX %fHZ",
    	switchArr[template],
    	shiftXHz, shiftYdB,deltaXHz);
 
    total += NumFmtdBytes();
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;       
}



/*===========================================================================*/
/* Function: DT Template Edit                                                */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_template_editDT (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLeveldB)
{
  	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;   

    if (avopt66_invalidViInt16Range (operation, 0, 2))
        return VI_ERROR_PARAMETER2;

	switch ( operation )
	{
	case 0: /* Copy */
		Fmt (wrtBuf, "%s<DTSTMPLCP");	
		break;
	case 1: /* entry */
		if (avopt66_invalidViReal64Range (templateFreqHz, -100.0e+9, 100.0e+9))
        	return VI_ERROR_PARAMETER3;
    	if (avopt66_invalidViReal64Range (templateLeveldB, -200, 100))
        	return VI_ERROR_PARAMETER4;
        Fmt (wrtBuf, "%s<DTSTMPLED %fHZ,%fDB",templateFreqHz,templateLeveldB);
		break;
	case 2: /* Init */
		Fmt (wrtBuf, "%s<DTSTMPLCLR");   
		break;
	}

    total += NumFmtdBytes();	
	
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;
}

/*===========================================================================*/
/* Function: DT Marker Edit                                                  */
/* Purpose:  This function sets the parameters for Due to Transient			 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_marker_editDT (ViSession instrumentHandle,
                                        ViInt16 operation, ViInt16 refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;    

	if (avopt66_invalidViInt16Range (operation, 0, 3))
        return VI_ERROR_PARAMETER2;

	switch (operation)
	{
		case 0: /*  STD */
			Fmt (wrtBuf, "%s<DTSMKRCP");	
			break;
			
		case 1: /* Entry */
			if (avopt66_invalidViBooleanRange(offsetMkrType))
        		return VI_ERROR_PARAMETER5;
    		if (avopt66_invalidViReal64Range(offsetFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER6;
    		if (avopt66_invalidViReal64Range(bandWidthHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER7;
    		if (avopt66_invalidViReal64Range(limitPowerdB, -200, 100))
        		return VI_ERROR_PARAMETER8;			
			
           	Fmt (wrtBuf, "%s<DTSMKRED %dENT,%fHZ,%fHZ,%fDB",offsetMkrType,offsetFreqHz,bandWidthHz,limitPowerdB);
			
			break;
		case 2:	/* Init */
			Fmt (wrtBuf, "%s<DTSMKRCLR");
			break;
		case 3: /* Ref */
			if (avopt66_invalidViInt16Range(refMarkerType, 0, 2))
        		return VI_ERROR_PARAMETER3;
    		if (avopt66_invalidViReal64Range (refBandWidthHz, 0, 1.0e+9))
        		return VI_ERROR_PARAMETER4;           	

			Fmt (wrtBuf, "%s<DTSMKRCLR; DTSMKRED %dENT,0HZ,%fHZ,0DB",
				refMarkerType,refBandWidthHz);

			break;
	}
	total += NumFmtdBytes();
	
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;      
}

/*===========================================================================*/
/* Function: DM Parameter Set                                                */
/* Purpose:  This function sets the parameters for Due to modulation		 */
/*	measurement.			                                                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_parameter_setDM (ViSession instrumentHandle,
                                          ViBoolean settoSTD,
                                          ViBoolean freqSetting,
                                          ViInt16 detector, ViInt16 result,
                                          ViBoolean refPower,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViBoolean judgement,
                                          ViReal64 templateLimit,
                                          ViReal64 symbolRate1T,
                                          ViReal64 rolloffFactor)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;
    
    if (avopt66_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<DTMSETSTD");	
	   total += NumFmtdBytes();            
	}
	else
	{
   		if (avopt66_invalidViBooleanRange (freqSetting))
        	return VI_ERROR_PARAMETER3;       
    	if (avopt66_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER4;
    	if (avopt66_invalidViInt16Range (result, 0, 2))
        	return VI_ERROR_PARAMETER5;
  		if (avopt66_invalidViBooleanRange (refPower))
        	return VI_ERROR_PARAMETER6;           	
   		if (avopt66_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER7;
   		if (avopt66_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_PARAMETER8; 
   		if (avopt66_invalidViBooleanRange (judgement))
        	return VI_ERROR_INSTR_PARAMETER9;  
 		if (avopt66_invalidViReal64Range(symbolRate1T,1,1e9))
        	return VI_ERROR_INSTR_PARAMETER10; 
		if (avopt66_invalidViReal64Range(rolloffFactor,0.01,0.99))
        	return VI_ERROR_INSTR_PARAMETER11;
 
        Fmt (wrtBuf, "%s<DTM%s; DTM%s; DTM%s; DTM%s; DTM%s; DTMTMPLPW %s; DTMTMPLBTM %fDB; DTMJDG %s;DTMSYMRT %fHZ; DTMRFACT %f",
        	FreqSetArr[freqSetting], DetArr[detector], dtsResultArr[result],
        	refMkrArr[refPower], LvlUnitArr[displayUnit],switchArr[TMPLCoupleofPWR],
			templateLimit, switchArr[judgement],symbolRate1T,rolloffFactor);
    	total += NumFmtdBytes();	
	}
        
 
    
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;	    	    
}

/*===========================================================================*/
/* Function: DM Template Set                                                 */
/* Purpose:  This function sets the parameters for Due to modulation		 */
/*	measurement. This function sets for Template.			                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_template_setDM (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz)
{
  	ViUInt32 i,retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;    

    if (avopt66_invalidViBooleanRange (template))
       	return VI_ERROR_PARAMETER2;
    if (avopt66_invalidViReal64Range (shiftXHz, -8.0e+9, 8.0e+9))
       	return VI_ERROR_PARAMETER3;   
    if (avopt66_invalidViReal64Range (shiftYdB, -200.0, 100))
       	return VI_ERROR_PARAMETER4; 
	if (avopt66_invalidViReal64Range (deltaXHz, -100e+6, 100e+6 ))
       	return VI_ERROR_PARAMETER5; 

    Fmt (wrtBuf, "%s<DTMTMPL %s; DTMTMPLSX %fHZ; DTMTMPLSY %fDB; DTMTMPLDX %fHZ",
    	switchArr[template], shiftXHz, shiftYdB, deltaXHz);
 
    total += NumFmtdBytes();
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;           
}

/*===========================================================================*/
/* Function: DM Template Edit                                                */
/* Purpose:  This function edets the parameters for Due to modulation		 */
/*	measurement. This function sets for Template.			                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_template_editDM (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLeveldB)
{
   	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;   

    if (avopt66_invalidViInt16Range (operation, 0, 2))
        return VI_ERROR_PARAMETER2;

	switch ( operation )
	{
	case 0: /* Copy */
		Fmt (wrtBuf, "%s<DTMTMPLCP");	
		break;
	case 1: /* entry */
		if (avopt66_invalidViReal64Range (templateFreqHz, -100.0e+9, 100.0e+9))
        	return VI_ERROR_PARAMETER3;
    	if (avopt66_invalidViReal64Range (templateLeveldB, -200, 100))
        	return VI_ERROR_PARAMETER4;
        Fmt (wrtBuf, "%s<DTMTMPLED %fHZ,%fDB",templateFreqHz,templateLeveldB);
		break;
	case 2: /* Init */
		Fmt (wrtBuf, "%s<DTMTMPLCLR");   
		break;
	}

    total += NumFmtdBytes();	
	
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;   
}

/*===========================================================================*/
/* Function: DM Marker Table Edit  											 */
/* Purpose:  This function sets the parameters for Due to modulation		 */
/*	measurement. This function sets for Template.			                 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_marker_editDM (ViSession instrumentHandle,
                                        ViInt16 operation,
                                        ViBoolean refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;    

	if (avopt66_invalidViInt16Range (operation, 0, 3))
        return VI_ERROR_PARAMETER2;

	switch (operation)
	{
		case 0: /*  STD */
			Fmt (wrtBuf, "%s<DTMMKRCP");	
			break;
			
		case 1: /* Entry */
			if (avopt66_invalidViBooleanRange (offsetMkrType))
        		return VI_ERROR_PARAMETER5;
    		if (avopt66_invalidViReal64Range(offsetFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER6;
    		if (avopt66_invalidViReal64Range(bandWidthHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER7;
    		if (avopt66_invalidViReal64Range(limitPowerdB, -200, 100))
        		return VI_ERROR_PARAMETER8;			
			
			Fmt (wrtBuf, "%s<DTMMKRED %d,%fHZ,%fHZ,%fDB",offsetMkrType,offsetFreqHz,bandWidthHz,limitPowerdB);

			break;
		case 2:	/* Init */
			Fmt (wrtBuf, "%s<DTMMKRCLR");
			break;
		case 3: /* Ref */
			if (avopt66_invalidViInt16Range (refMarkerType, 0, 2))
        		return VI_ERROR_PARAMETER3;
    		if (avopt66_invalidViReal64Range (refBandWidthHz, 0, 1.0e+9))
        		return VI_ERROR_PARAMETER4; 
			Fmt (wrtBuf, "%s<DTMMKRCLR; DTMMKRED %dENT,0HZ,%fHZ,0DB",refMarkerType,refBandWidthHz);
			break;
	}
	total += NumFmtdBytes();
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;          
}


/*===========================================================================*/
/* Function: InBand Spurious Parameter Set                                   */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/


ViStatus CVIFUNC avopt66_parameter_setINSPR (ViSession instrumentHandle,
                                             ViBoolean settoSTD,
                                             ViBoolean freqSetting,
                                             ViInt16 detector,
                                             ViBoolean refPower,
                                             ViReal64 peakMkrYDelta,
                                             ViInt16 result, ViInt16 displayUnit,
                                             ViBoolean TMPLCoupleofPWR,
                                             ViReal64 templateLimit,
                                             ViBoolean judgement)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;
    
    if (avopt66_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<SPRSETSTD");	
	}
	else
	{
   		if (avopt66_invalidViBooleanRange (freqSetting))
        	return VI_ERROR_PARAMETER3;       
    	if (avopt66_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER4;
  		if (avopt66_invalidViBooleanRange (refPower))
        	return VI_ERROR_PARAMETER5; 
    	if (avopt66_invalidViReal64Range (peakMkrYDelta, 0.1, 10.0))
        	return VI_ERROR_PARAMETER6;           	
   		if (avopt66_invalidViInt16Range (result, 0, 2))
        	return VI_ERROR_PARAMETER7;        	
   		if (avopt66_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER8;
   		if (avopt66_invalidViBooleanRange (TMPLCoupleofPWR))
        	return VI_ERROR_INSTR_PARAMETER9; 
   		if (avopt66_invalidViBooleanRange (judgement))
        	return VI_ERROR_INSTR_PARAMETER11; 

        Fmt (wrtBuf, "%s<SPR%s; SPR%s;SPRPKMKY %fENT; SPR%s; SPR%s; SPR%s;SPRTMPLPW %s;SPRTMPLBTM %fDB; SPRJDG %s",
        	FreqSetArr[freqSetting], DetArr[detector],peakMkrYDelta, dtsResultArr[result],
        	refMkrArr[refPower], LvlUnitArr[displayUnit],switchArr[TMPLCoupleofPWR],
			templateLimit, switchArr[judgement]);
	}
        
    total += NumFmtdBytes();
    
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;	    	        
}

/*===========================================================================*/
/* Function: InBand Spurious Template Set                                    */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/

ViStatus CVIFUNC avopt66_template_setINSPR (ViSession instrumentHandle,
                                            ViBoolean template,
                                            ViReal64 shiftXHz, ViReal64 shiftYdB,
                                            ViReal64 deltaXHz)
{
  	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE],wrtBuf2[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;    


    if (avopt66_invalidViBooleanRange (template))
       	return VI_ERROR_PARAMETER2;
    if (avopt66_invalidViReal64Range (shiftXHz, -8.0e+9, 8.0e+9))
       	return VI_ERROR_PARAMETER3;   
    if (avopt66_invalidViReal64Range (shiftYdB, -200.0, 100))
       	return VI_ERROR_PARAMETER4; 
	if (avopt66_invalidViReal64Range (deltaXHz, -100e+6, 100e+6))
       	return VI_ERROR_PARAMETER5; 

    Fmt (wrtBuf, "%s<SPRTMPL %s; SPRTMPLSX %fHZ; SPRTMPLSY %fDB; SPRTMPLDX %fHZ",switchArr[template],
    	shiftXHz, shiftYdB,deltaXHz);
 
    total += NumFmtdBytes();
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;	        
}

/*===========================================================================*/
/* Function: InBand Spurious Template Edit                                   */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_template_editINSPR (ViSession instrumentHandle,
                                             ViInt16 operation,
                                             ViReal64 templateFreqHz,
                                             ViReal64 templateLeveldB)
{
   	ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;   

    if (avopt66_invalidViInt16Range (operation, 0, 2))
        return VI_ERROR_PARAMETER2;

	switch ( operation )
	{
	case 0: /* Copy */
		Fmt (wrtBuf, "%s<SPRTMPLCP");	
		break;
	case 1: /* entry */
		if (avopt66_invalidViReal64Range (templateFreqHz, -100.0e+9, 100.0e+9))
        	return VI_ERROR_PARAMETER3;
    	if (avopt66_invalidViReal64Range (templateLeveldB, -200, 100))
        	return VI_ERROR_PARAMETER4;
        Fmt (wrtBuf, "%s<SPRTMPLED %fHZ,%fDB",templateFreqHz,templateLeveldB);
		break;
	case 2: /* Init */
		Fmt (wrtBuf, "%s<SPRTMPLCLR");   
		break;
	}

    total += NumFmtdBytes();	
	
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;   
}    


/*===========================================================================*/
/* Function: InBand Spurious Marker Set                                      */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_marker_editINSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean refMarkerType,
                                           ViReal64 refBandWidthHz,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz,
                                           ViReal64 limitPowerdB)
{
 	ViUInt32 i, retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;    

	if (avopt66_invalidViInt16Range (operation, 0, 3))
        return VI_ERROR_PARAMETER2;

	switch (operation)
	{
		case 0: /*  STD */
			Fmt (wrtBuf, "%s<SPRMKRCP");	
			break;
			
		case 1: /* Entry */
    			if (avopt66_invalidViReal64Range(startFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER5;
    			if (avopt66_invalidViReal64Range(stopFreqHz, 0, 100.0e+9))
        		return VI_ERROR_PARAMETER6;
    			if (avopt66_invalidViReal64Range(limitPowerdB, -200, 100))
        		return VI_ERROR_PARAMETER7;

            	Fmt (wrtBuf, "%s<SPRMKRED 0,%fHZ,%fHZ,%fDB", startFreqHz, stopFreqHz, limitPowerdB);
			break;
		case 2:	/* Init */
			Fmt (wrtBuf, "%s<SPRMKRCLR");
			break;
		case 3: /* Ref */
			if (avopt66_invalidViBooleanRange (refMarkerType))
        		return VI_ERROR_PARAMETER3;
    		if (avopt66_invalidViReal64Range (refBandWidthHz, 0, 1.0e+9))
        		return VI_ERROR_PARAMETER4;     
			Fmt (wrtBuf, "%s<SPRMKRCLR; SPRMKRED %d,0HZ,%fHZ,0DB",refMarkerType,refBandWidthHz);
			break;
	}
	
	total += NumFmtdBytes();
	
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;    
}

/*===========================================================================*/
/* Function: OutBand Spurious Parameter Set                                  */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_parameter_setOUTSPR (ViSession instrumentHandle,
                                              ViBoolean settoSTD,
                                              ViInt16 detector,
                                              ViReal64 peakMkrYDelta,
                                              ViInt16 displayUnit,
                                              ViBoolean judgement,
                                              ViBoolean preselector)
{
	ViUInt32 total = 0;       
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;
    
    if (avopt66_invalidViBooleanRange (settoSTD))
        return VI_ERROR_PARAMETER2;

	if (settoSTD == VI_ON)
	{
		Fmt (wrtBuf, "%s<FDSSETSTD");	
	}
	else
	{
	
    	if (avopt66_invalidViInt16Range (detector, 0, 3))
        	return VI_ERROR_PARAMETER3;
    	if (avopt66_invalidViReal64Range (peakMkrYDelta, 0.1, 10.0))
        	return VI_ERROR_PARAMETER4;           	
   		if (avopt66_invalidViInt16Range (displayUnit, 0, 2))
        	return VI_ERROR_PARAMETER5;
   		if (avopt66_invalidViBooleanRange (judgement))
        	return VI_ERROR_PARAMETER6; 
   		if (avopt66_invalidViBooleanRange (preselector))
        	return VI_ERROR_PARAMETER7; 

        Fmt (wrtBuf, "%s<FDS%s;FDSPKMKY %fENT; FDS%s; FDSJDG %s; FDSPRE %s",
        	DetArr[detector], peakMkrYDelta,
        	LvlUnitArr[displayUnit], switchArr[judgement],preselArr[preselector]);
	}
        
    total += NumFmtdBytes();
    
    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;    
}

/*===========================================================================*/
/* Function: OutBand Spurious Table Set                                      */
/* Purpose:  This function sets the parameters for In Band Spurious			 */
/*           measurement.													 */
/*===========================================================================*/

ViStatus CVIFUNC avopt66_table_editOUTSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViInt16 tableNumber,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz, ViReal64 RBW,
                                           ViReal64 VBW, ViReal64 sweepTime,
                                           ViReal64 limitLevel)
{
 	ViUInt32 total = 0;       
    ViUInt32 i, retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

	
	if (avopt66_invalidViInt16Range (operation, 0, 4))
        return VI_ERROR_PARAMETER2;
	if (avopt66_invalidViInt16Range (tableNumber, 1, 3))
        return VI_ERROR_PARAMETER3;

    switch (operation)
    {
    	case 0: /* Load */
    		Fmt (wrtBuf, "%s<FDSTBL %d; FDSLD",tableNumber);
    		break;
    	case 1: /* Save */
    		Fmt (wrtBuf, "%s<FDSTBL %d; FDSSV",tableNumber);
    		break;
    	case 2: /* Sel */
    		Fmt (wrtBuf, "%s<FDSTBL %d",tableNumber);
    		break;
    	case 3: /* Edit */
    		if (avopt66_invalidViReal64Range (startFreqHz, -200e+6, 8e+9))
        		return VI_ERROR_PARAMETER4;
			if (avopt66_invalidViReal64Range (stopFreqHz, 0.0, 8.2e+9))
        		return VI_ERROR_PARAMETER5;
			if (avopt66_invalidViReal64Range (RBW, 10.0, 10e+6))
        		return VI_ERROR_PARAMETER6;
        	if (avopt66_invalidViReal64Range (VBW, 1.0, 10e+6))
        		return VI_ERROR_PARAMETER7;
        	if (avopt66_invalidViReal64Range (sweepTime, 1e-6, 1000))
        		return VI_ERROR_PARAMETER8;        	
        	if (avopt66_invalidViReal64Range (limitLevel, -155, 65))
        		return VI_ERROR_INSTR_PARAMETER9;

     		Fmt (wrtBuf, "%s<FDSTBLED %fHZ,%fHZ, %fHZ, %fHZ, %fSC,%fDB",
     			startFreqHz,stopFreqHz,RBW,VBW,sweepTime,limitLevel);
    		break;
    	case 4: /* Init */
    		Fmt (wrtBuf, "%s<FDSCLR");
    	
    		break;
    }
   total += NumFmtdBytes();
	
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;    
}

/*===========================================================================*/
/* Function: Modulation Parameter Set 	                                     */
/* Purpose:  This function sets the parameters for Modulation   			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_parameter_setMOD (ViSession instrumentHandle,
                                           ViInt16 triggerSource,
                                           ViBoolean burstSwitch,
                                           ViReal64 searchLevel,
                                           ViBoolean trgSlope,
                                           ViInt16 triggerLevel,
                                           ViBoolean extTrgDelay,
                                           ViInt16 slotNumber,
                                           ViReal64 delayTimes,
                                           ViBoolean bitSequence)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if ( avopt66_invalidViInt16Range(triggerSource,0,2))
        return VI_ERROR_PARAMETER2;
    if (avopt66_invalidViBooleanRange(burstSwitch))
        return VI_ERROR_PARAMETER3;
    if (avopt66_invalidViReal64Range(searchLevel, -40.0, -5.0))
        return VI_ERROR_PARAMETER4;
    if (avopt66_invalidViBooleanRange(trgSlope))
        return VI_ERROR_PARAMETER5;
    if (avopt66_invalidViInt16Range(triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER6;
    if (avopt66_invalidViBooleanRange(extTrgDelay))
        return VI_ERROR_PARAMETER7;
    if (avopt66_invalidViInt16Range(slotNumber, 0, 7))
        return VI_ERROR_PARAMETER8;
    if (avopt66_invalidViReal64Range(delayTimes, 0, 5e-3))
        return VI_ERROR_INSTR_PARAMETER9;    
	if (avopt66_invalidViBooleanRange(bitSequence))
        return VI_ERROR_INSTR_PARAMETER10;

    Fmt (wrtBuf, "%s<MODTRG %s; MODTRGBRST %s; MODBRSTLVL %f; MODTRGSLP %s;MODTRGLVL %d",
    	modTrgArr[triggerSource],
    	switchArr[burstSwitch],
    	searchLevel,
    	slopeArr[trgSlope],
    	triggerLevel);
    
     total += NumFmtdBytes();   
    
    if(extTrgDelay==1)
    {
    	Fmt (wrtBuf, "%s[a]<; MODTRGSLT %d",
    		  slotNumber);
    }
    else
    {
    	Fmt (wrtBuf, "%s[a]<; MODTRGDLY %f",
    		 delayTimes);
    }
    
    total += NumFmtdBytes();   
    Fmt (wrtBuf, "%s[a]<; FMBITSEQ %s",
    	fmBitSqArr[bitSequence]
    	);

    total += NumFmtdBytes();
	
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;     
}

/*===========================================================================*/
/* Function: Modulation Lockup Time Graph Scale Set                          */
/* Purpose:  This function sets the parameters for Modulation Lockup Time	 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_lockupTimGrpScl_setMOD (ViSession instrumentHandle,
                                                 ViReal64 XSpan, ViReal64 XStart,
                                                 ViInt16 YScale)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

	if (avopt66_invalidViReal64Range(XSpan, 0, 20.0e-3))
        return VI_ERROR_PARAMETER2;
    if (avopt66_invalidViReal64Range(XStart, 0, 20.0e-3))
        return VI_ERROR_PARAMETER3;
    if (avopt66_invalidViInt16Range(YScale, 0, 4))
        return VI_ERROR_PARAMETER4;
        
    Fmt (wrtBuf, "%s<LKUPXSP %fSC; LKUPXST %fSC; LKUPYSCL %s",     
    	XSpan, XStart, lkupYSclArr[YScale]);    
    total += NumFmtdBytes();
	
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;  
}

/*===========================================================================*/
/* Function: Lockup time Parameter Set                                       */
/* Purpose:  This function sets the parameters for Lockup Time  			 */
/*           measurement.													 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_lockupTimParam_setMOD (ViSession instrumentHandle,
                                                ViBoolean freqRange,
                                                ViReal64 analyzeLength,
                                                ViInt16 triggerSource,
                                                ViBoolean triggerSlop,
                                                ViInt16 triggerLevel,
                                                ViBoolean extTrgDelay,
                                                ViInt16 slotNumber,
                                                ViReal64 triggerDelay,
                                                ViReal64 limit)
{
    ViUInt32 retCnt = 0;
    ViUInt32 total = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if (avopt66_invalidViBooleanRange(freqRange))
        return VI_ERROR_PARAMETER2;
    if (avopt66_invalidViReal64Range(analyzeLength, 0.1e-3, 20e-3))
        return VI_ERROR_PARAMETER3;
    if (avopt66_invalidViInt16Range(triggerSource, 0, 2))
        return VI_ERROR_PARAMETER4;
    if (avopt66_invalidViBooleanRange(triggerSlop))
        return VI_ERROR_PARAMETER5;
    if (avopt66_invalidViInt16Range(triggerLevel, 0, 100))
        return VI_ERROR_PARAMETER6;
    if (avopt66_invalidViBooleanRange(extTrgDelay))
        return VI_ERROR_PARAMETER7;
    if (avopt66_invalidViInt16Range(slotNumber, 0, 5))
        return VI_ERROR_PARAMETER8;
    if (avopt66_invalidViReal64Range(triggerDelay, 0, 5e-3))
        return VI_ERROR_INSTR_PARAMETER9;    
	if (avopt66_invalidViReal64Range(limit, 0, 1.0e+6))
        return VI_ERROR_INSTR_PARAMETER10;
        

    Fmt (wrtBuf, "%s<LKUPFRNG %s; LKUPALEN %fSC; LKUPTRG %s; LKUPTRGSLP %s; LKUPTRGLVL %dENT",
    	lkupFRanArr[freqRange],
    	analyzeLength,
    	modTrgArr[triggerSource],
    	slopeArr[triggerSlop],
    	triggerLevel
    	);

	total += NumFmtdBytes();

	if (extTrgDelay)
	{
		Fmt (wrtBuf, "%s[a]<; LKUPTRGSLT %dENT",
			slotNumber);
	}
	else
	{
		Fmt (wrtBuf, "%s[a]<; LKUPTRGDLY %fSC",
			triggerDelay);
	}

	total += NumFmtdBytes();  

	Fmt (wrtBuf, "%s[a]<; LKUPLMT %fHZ",
		limit);

    total += NumFmtdBytes();
	
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, total, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;     
}

/*===========================================================================*/
/* Function: Averaging                                                       */
/* Purpose:  This function sets the averaging times of Burst Envelope		 */
/*           Spectrum, Gated Output Power, On/Off Ratio, OBW,  Channel Power,*/
/*           In Band Spurious, Waveform Quality, Code Domain Power, TX Power */
/*           functions.														 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_average_set (ViSession instrumentHandle,
                                      ViInt16 function, ViInt16 AVGTimes)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if (avopt66_invalidViInt16Range (function, 0, 11))
        return VI_ERROR_PARAMETER2;
    if (avopt66_invalidViInt16Range (AVGTimes, 1, 999))
        return VI_ERROR_PARAMETER3;

    Fmt (wrtBuf, "%s<%s %d", avgFunctionArr[function], AVGTimes);

    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;    
}


/*===========================================================================*/
/* Function: Auto Level Set                                                  */
/* Purpose:  This function is used for Auto Level set in selected meas mode. */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_execute_auto_lvl (ViSession instrumentHandle,
                                           ViInt16 function)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if (avopt66_invalidViInt16Range (function, 0, 9))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", autoLvlFunctionArr[function]);

    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;    
}


/*===========================================================================*/
/* Function: DC Cal					                                         */
/* Purpose:  This function execute DC Cal.									 */
/*===========================================================================*/

ViStatus CVIFUNC avopt66_act_DcCal (ViSession instrumentHandle)
{
    ViUInt32 retCnt = 0;
    ViStatus avopt66_status = VI_SUCCESS;
    
    if ((avopt66_status = viWrite (instrumentHandle, "CLDC", 4, &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;    
}

/*===========================================================================*/
/* Function: Start Measurement		                                         */
/* Purpose:  This function starts desired measurement mode of TRANSIENT		 */
/*           section.														 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_measStart_act (ViSession instrumentHandle,
                                        ViInt16 function)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if (avopt66_invalidViInt16Range (function, 0, 12))
        return VI_ERROR_PARAMETER2;

    Fmt (wrtBuf, "%s<%s", funcMeasureArr[function]);

    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;    
}


/*===========================================================================*/
/* Function: Graph Type Selection(FM Deviation).                             */
/* Purpose:  This function is used for graph type selection.				 */
/*===========================================================================*/

ViStatus CVIFUNC avopt66_graphTyp_selFMD (ViSession instrumentHandle,
                                          ViInt16 graphType, ViInt16 startBit)
{
    ViUInt32 retCnt = 0;
    ViChar wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if (avopt66_invalidViInt16Range (graphType, 0, 4))
        return VI_ERROR_PARAMETER2;
        
    if (avopt66_invalidViInt16Range (startBit, -100, 338))
        return VI_ERROR_PARAMETER2;
        
    Fmt (wrtBuf, "%s<STTBIT %d; %s",startBit,gTypFMArr[graphType]);

    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes (), &retCnt)) < 0)
        return avopt66_status;

    return avopt66_status;     
}


/*===========================================================================*/
/* Function: Check meas mode		                                         */
/* Purpose:  This function returns the mode of instrument.(CW/Transient)     */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_check_measMode (ViSession instrumentHandle,
                                         ViInt16 *returnmode)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;
    

    if ((avopt66_status = viWrite (instrumentHandle, "SETFUNC?", 8, &retCnt)) < 0)
        return avopt66_status;
    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%d[b2]", returnmode) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt66_status;	       
    
}

/*===========================================================================*/
/* Function: Time Domain Power Result                                        */
/* Purpose:  This function returns the result of power measurement in        */
/*           the time domain.												 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_data_TDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if ((avopt66_status = viWrite (instrumentHandle, "TDPMEAS?", 8, &retCnt)) < 0)
        return avopt66_status;
    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%f[e],%d[b2]", powerLeveldBm,judgement) < 2)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt66_status;	    
}

/*===========================================================================*/
/* Function: On/Off Ratio                                                    */
/* Purpose:  This function returns the result of On/Off ratio measurement of */
/*           the burst signal in the time domain.							 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_data_OnOffRatio (ViSession instrumentHandle,
                                          ViReal64 *ONLevel, ViReal64 *OFFLevel,
                                          ViReal64 *ONOFFRaito,
                                          ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if ((avopt66_status = viWrite (instrumentHandle, "OORMEAS?", 8, &retCnt)) < 0)
        return avopt66_status;
    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%f[e],%f[e],%f[e],%d[b2]", ONLevel, OFFLevel, ONOFFRaito, judgement) < 4)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt66_status;    
}


/*===========================================================================*/
/* Function: TD Spurious Data                                                */
/* Purpose:  This function returns the result of Time Domain Spurious		 */
/*           measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_data_spuriTD (ViSession instrumentHandle,
                                       ViInt16 *tableSize,
                                       ViReal64 frequencyHz[], 
                                       ViReal64 level[],
                                       ViInt16 judgement[])
{
    ViUInt32 i, retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if ((avopt66_status = viWrite (instrumentHandle, "TDSMEAS?", 8, &retCnt)) < 0)
        return avopt66_status;
    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%d[b2]", tableSize) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	
	for (i=0;i<*tableSize;i++) {
	    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt66_status;
	   	if (Scan (rdBuf, "%s>%f[e],%f[e],%d[b2]", &frequencyHz[i], &level[i], &judgement[i]) < 3)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
   	
    return avopt66_status;
}

/*===========================================================================*/
/* Function: F Domain Power Result                                           */
/* Purpose:  This function returns the result of power measurement in        */
/*           the time domain.												 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_data_FDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if ((avopt66_status = viWrite (instrumentHandle, "FDPMEAS?", 8, &retCnt)) < 0)
        return avopt66_status;
    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%f[e],%d[b2]", powerLeveldBm,judgement) < 2)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt66_status;	        
}

/*===========================================================================*/
/* Function: OBW Data                                                        */
/* Purpose:  This function returns the result of occupied bandwidth			 */
/*           measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_data_OBW (ViSession instrumentHandle, ViReal64 *OBWHz,
                                   ViReal64 *lowFreqHz, ViReal64 *hiFreqHz,
                                   ViInt16 *judgement)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if ((avopt66_status = viWrite (instrumentHandle, "OBWMEAS?", 8, &retCnt)) < 0)
        return avopt66_status;
    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%f[e],%f[e],%f[e],%d[b2]", OBWHz, lowFreqHz, hiFreqHz, judgement) < 4)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt66_status;    
}

/*===========================================================================*/
/* Function: Due to Transient Data                                           */
/* Purpose:  This function returns the result of Due to Transient			 */
/*			 measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_data_duetoTRN (ViSession instrumentHandle,
                                        ViInt16 *numberofACP,
                                        ViReal64 ACPDatadBc[],
                                        ViInt16 judgement[])
{
    ViUInt32 i,retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if ((avopt66_status = viWrite (instrumentHandle, "DTSMEAS?", 8, &retCnt)) < 0)
        return avopt66_status;

    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofACP) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *numberofACP; i++)
	{
	    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt66_status;
	   	if (Scan (rdBuf, "%s>%f[e],%d[b2]", &ACPDatadBc[i], &judgement[i]) < 2)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;		
	}
	
    return avopt66_status;    
}

/*===========================================================================*/
/* Function: Due to Modulation Data                                          */
/* Purpose:  This function returns the result of Due to Modulation			 */
/*			 measurement function in the frequency domain.					 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_data_duetoMOD (ViSession instrumentHandle,
                                        ViInt16 *numberofACP, ViReal64 ACPdBc[],
                                        ViInt16 ACPJudge[])
{
    ViUInt32 i,retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if ((avopt66_status = viWrite (instrumentHandle, "DTMMEAS?", 8, &retCnt)) < 0)
        return avopt66_status;
    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofACP) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *numberofACP; i++)
	{
	    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt66_status;
	   	if (Scan (rdBuf, "%s>%f[e],%d[b2]", &ACPdBc[i], &ACPJudge[i]) < 2)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;		
	}
    return avopt66_status;    	    
}

/*===========================================================================*/
/* Function: In Band Spurious                                                */
/* Purpose:  This function returns the result of spurious measurement. 		 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_data_INSPR (ViSession instrumentHandle,
                                     ViInt16 *numberofPoint,
                                     ViReal64 frequencyHz[], ViReal64 leveldBm[],
                                     ViInt16 judge[])
{
    ViUInt32 i, retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if ((avopt66_status = viWrite (instrumentHandle, "SPRMEAS?", 8, &retCnt)) < 0)
        return avopt66_status;
        
    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofPoint) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	
	for (i=0;i<*numberofPoint;i++) {
	    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt66_status;
	   	
	   	if (Scan (rdBuf, "%s>%f[e],%f[e],%d[b2]", &frequencyHz[i], &leveldBm[i], &judge[i]) < 3)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
    return avopt66_status;
}

/*===========================================================================*/
/* Function: Out Band Spurious                                               */
/* Purpose:  This function returns the result of spurious measurement. 		 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_data_spuriOB (ViSession instrumentHandle,
                                       ViInt16 *numberofPoint,
                                       ViReal64 frequencyHz[],
                                       ViReal64 leveldBm[], ViInt16 judgement[])
{
    ViUInt32 i, retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if ((avopt66_status = viWrite (instrumentHandle, "FDSMEAS?", 8, &retCnt)) < 0)
        return avopt66_status;
        
    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%d[b2]", numberofPoint) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	   	
	for (i=0;i<*numberofPoint;i++) {
	    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   		    return avopt66_status;
	   	
	   	if (Scan (rdBuf, "%s>%f[e],%f[e],%d[b2]", &frequencyHz[i], &leveldBm[i], &judgement[i]) < 3)
   		   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
   	}
    return avopt66_status;		
}


/*===========================================================================*/
/* Function: TX Power	                                                     */
/* Purpose:  This function returns the result of TX power measurement. 		 */
/*===========================================================================*/

ViStatus CVIFUNC avopt66_data_TxPower (ViSession instrumentHandle,
                                       ViReal64 *txPowerdBm, ViReal64 *txPowerW,
                                       ViReal64 *peakTxPowerdBm,
                                       ViReal64 *posofpeakTxPower,
                                       ViReal64 *peakTxPowerW)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if ((avopt66_status = viWrite (instrumentHandle, "TXPWR?", 6, &retCnt)) < 0)
        return avopt66_status;
    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%f[e],%f[e],%f[e],%f[e],%f[e]",
   		txPowerdBm,txPowerW, peakTxPowerdBm,posofpeakTxPower,peakTxPowerW) < 5)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt66_status;
}



/*===========================================================================*/
/* Function: Lockup Time Data	                                             */
/* Purpose:  This function returns the result of Lockup Time measurement. 	 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_data_LockupTime (ViSession instrumentHandle,
                                          ViReal64 *lockupTime)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if ((avopt66_status = viWrite (instrumentHandle, "LKUPTM?", 7, &retCnt)) < 0)
        return avopt66_status;
    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%f[e]", lockupTime) < 1)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt66_status;    
}
/*===========================================================================*/
/* Function: Lockup Time Graph Data	                                         */
/* Purpose:  This function returns the result of Lockup Time XY data Array.  */
/*===========================================================================*/

ViStatus CVIFUNC avopt66_data_graphLkupTim (ViSession instrumentHandle,
                                            ViInt16 *XDataSize, ViReal64 XData[],
                                            ViInt16 *YDataSize, ViReal64 YData[])
{
    ViUInt32 retCnt = 0;
    ViInt16 i;
    ViChar rdBuf[BUFFER_SIZE],wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

	/* X Data */
   	if ((avopt66_status = viWrite (instrumentHandle, "LGPHX?", 6, &retCnt)) < 0)
       	return avopt66_status;
    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%d[b2]", XDataSize) < 1 )
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *XDataSize; i++)
	{
	    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt66_status;
   	    	
   		if (Scan (rdBuf, "%s>%f[e]", &XData[i]) < 1)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
    
	/* Y Data */
   	if ((avopt66_status = viWrite (instrumentHandle, "LGPHY?", 6, &retCnt)) < 0)
       	return avopt66_status;
	
    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%d[b2]", YDataSize) < 1 )
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

	for (i = 0; i < *YDataSize; i++)
	{
	    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt66_status;
   	    	
   		if (Scan (rdBuf, "%s>%f[e]", &YData[i]) < 1)
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
	}
    
    return avopt66_status;
}

/*===========================================================================*/
/* Function: Freq Deviation Random Sequence Data							 */
/* Purpose:  This function returns the result of the Frequency Deviation	 */
/*			 measurement. Bit Sequence is set to RANDOM.					 */
/*===========================================================================*/

ViStatus CVIFUNC avopt66_dataFreqDevRandSeq (ViSession instrumentHandle,
                                             ViReal64 *frequencyError,
                                             ViReal64 *maxPeakDev,
                                             ViReal64 *posofMaxPeakDev,
                                             ViReal64 *minPeakDev,
                                             ViReal64 *posofMinPeakDev,
                                             ViReal64 *minMaxRaito,
                                             ViReal64 *txPowerdBm,
                                             ViReal64 *txPowerW,
                                             ViReal64 *peakTxPowerdBm,
                                             ViReal64 *posofPeakTxPower,
                                             ViReal64 *peakTxPowerW)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    if ((avopt66_status = viWrite (instrumentHandle, "FMDEV?", 6, &retCnt)) < 0)
        return avopt66_status;
    if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    return avopt66_status;

   	if (Scan (rdBuf, "%s>%f[e],%f[e],%f[e],%f[e],%f[e],%f[e],%f[e],%f[e],%f[e],%f[e],%f[e]",
   		frequencyError, maxPeakDev, posofMaxPeakDev,
   		minPeakDev, posofMinPeakDev, minMaxRaito,
   		txPowerdBm, txPowerW, peakTxPowerdBm,
   		posofPeakTxPower, peakTxPowerW) < 11)
   	   	return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

    return avopt66_status;
}

/*===========================================================================*/
/* Function: Freq Deviation Standard Data									 */
/* Purpose:  This function returns the Standard data of FM					 */
/*			 deviation measurement.											 */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_data_fmSTD (ViSession instrumentHandle,
                                     ViReal64 *frequencyError,
                                     ViReal64 *maxPeakDev,
                                     ViReal64 *posofMaxPeakDev,
                                     ViReal64 *avgPeakDeviHz,
                                     ViReal64 *frequencyDriftHz,
                                     ViReal64 *maxDriftRaito,
                                     ViReal64 *txPowerdBm, ViReal64 *txPowerW,
                                     ViReal64 *peakTxPowerdBm,
                                     ViReal64 *posofPeakTxPower,
                                     ViReal64 *peakTxPowerW)
{
    ViUInt32 retCnt = 0;
    ViChar rdBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

    	if ((avopt66_status = viWrite (instrumentHandle, "FMDEVSTD?", 9, &retCnt)) < 0)
        	return avopt66_status;
    	if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt66_status;

   		if (Scan (rdBuf, "%s>%f[e],%f[e],%f[e],%f[e],%f[e],%f[e],%f[e],%f[e],%f[e],%f[e],%f[e]",
			  frequencyError, maxPeakDev, posofMaxPeakDev,
			  avgPeakDeviHz, frequencyDriftHz, maxDriftRaito,
			  txPowerdBm, txPowerW, peakTxPowerdBm,
			  posofPeakTxPower, peakTxPowerW ) <11)
    		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;


    return avopt66_status;
}

/*===========================================================================*/
/* Function: Freq Deviation Graphic Data									 */
/* Purpose:  This function returns the graphic data of DECT frequency		 */
/*			 deviation measurement.											 */
/*===========================================================================*/

ViStatus CVIFUNC avopt66_dataGraphFreqDeviation (ViSession instrumentHandle,
                                                 ViInt16 startBit,
                                                 ViInt16 graphType,
                                                 ViInt16 *XDataSize,
                                                 ViReal64 XData[],
                                                 ViInt16 *YDataSize,
                                                 ViReal64 YData[],
                                                 ViInt16 *demodulatedDataSize,
                                                 ViInt32 demodulatedData[])
{
    ViUInt32 retCnt = 0;
    ViInt16 i;
    ViChar rdBuf[BUFFER_SIZE],wrtBuf[BUFFER_SIZE];
    ViStatus avopt66_status = VI_SUCCESS;

	/* Graph Type Select */
	if (avopt66_invalidViInt16Range (graphType, 0, 5))
		return VI_ERROR_PARAMETER2;
	
    Fmt (wrtBuf, "%s<FMGTYP %s", FreqDevGraphArr[graphType]);

    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt66_status;
        
    Delay (1.0);
    
    
    /* Set Start Bit */
    
    Fmt (wrtBuf, "%s<STTBIT %d; SI", startBit);

    if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
        return avopt66_status;

    if (graphType > 3 )
    {
    	Delay (1.5);
    }
    else
    {
    	Delay (1.0);
    }
    
    /* Get Data */

 	Fmt (wrtBuf, "%s<%s", FMDevGqueArr[graphType]);            
	if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
       		return avopt66_status;
 
 
    if (graphType == 2)/* Demodulated Data */   
    {
    	if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt66_status;

   		if (Scan (rdBuf, "%s>%d[b2]", demodulatedDataSize) < 1 )
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

		for (i = 0; i < *demodulatedDataSize; i++)
		{
	    	if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    		return avopt66_status;
   	    	
   			if (Scan (rdBuf, "%s>%d", &demodulatedData[i]) < 1) /* 32bit INT */
   	   			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		}
	}
	else
	{
		/* X Data */
    	if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt66_status;

   		if (Scan (rdBuf, "%s>%d[b2]", XDataSize) < 1 )
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

		for (i = 0; i < *XDataSize; i++)
		{
	    	if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    		return avopt66_status;
   	    	
   			if (Scan (rdBuf, "%s>%f[e]", &XData[i]) < 1)
   	   			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		}
    
		/* Y Data */

	 	Fmt (wrtBuf, "%s<%s", FMDevGqueYArr[graphType]);            
		if ((avopt66_status = viWrite (instrumentHandle, wrtBuf, NumFmtdBytes(), &retCnt)) < 0)
       		return avopt66_status;

    	if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    	return avopt66_status;

   		if (Scan (rdBuf, "%s>%d[b2]", YDataSize) < 1 )
   	   		return VI_ERROR_INSTR_INTERPRETING_RESPONSE;

		for (i = 0; i < *YDataSize; i++)
		{
	    	if ((avopt66_status = viRead (instrumentHandle, rdBuf, BUFFER_SIZE, &retCnt)) < 0)
   	    		return avopt66_status;
   	    	
   			if (Scan (rdBuf, "%s>%f[e]", &YData[i]) < 1)
   	   			return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
		}
	}

    return avopt66_status;
}

/*===========================================================================*/
/* Function: Close                                                           */
/* Purpose:  This function closes the instrument.                            */
/*===========================================================================*/
ViStatus CVIFUNC avopt66_close (ViSession instrSession)
{
    avopt66_instrRange instrPtr;
    ViSession rmSession;
    ViStatus avopt66_status = VI_SUCCESS;

    if ((avopt66_status = viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession)) < 0)
        return avopt66_status;
    if ((avopt66_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr)) < 0)
        return avopt66_status;
            
    if (instrPtr != NULL) 
        free (instrPtr);
    
    avopt66_status = viClose (instrSession);
    viClose (rmSession);

    return avopt66_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean avopt66_invalidViBooleanRange (ViBoolean val)
{
    return ((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Signed Integer Value Out Of Range - ViInt16               */
/* Purpose:  This function checks a short signed integer value to see if it  */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt66_invalidViInt16Range (ViInt16 val, ViInt16 min, ViInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt66_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Unsigned Char Value Out Of Range - ViUInt8                      */
/* Purpose:  This function checks an unsigned char value to see if it        */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt66_invalidViUInt8Range (ViUInt8 val, ViUInt8 min, ViUInt8 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Short Unsigned Integer Value Out Of Range - ViUInt16            */
/* Purpose:  This function checks a short unsigned integer value to see if it*/  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt66_invalidViUInt16Range (ViUInt16 val, ViUInt16 min, ViUInt16 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long unsigned integer value to see if it */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean avopt66_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Float) Value Out Of Range - ViReal32                      */
/* Purpose:  This function checks a real (float) value to see if it lies     */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avopt66_invalidViReal32Range (ViReal32 val, ViReal32 min, ViReal32 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean avopt66_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return ((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*================================================================================*/
/* Function: Wait On VISA Event                                                   */
/* Purpose:  This function waits for the occurrence of an enabled VISA Event. The */
/*           event type must be enabled before entering this function. Any        */
/*           programmatic commands, actions, or conditions, necessary to generate */
/*           the specified event type, must be satisified before entering this    */
/*           function. The function will wait for the specified timeout and then  */
/*           return. If the specified event is received then the function will    */
/*           return VI_SUCCESS (0), otherwise the status code for the generated   */
/*           error will be returned. In either case the function will disable the */
/*           event type and deallocate the event handle passed from viWaitOnEvent */
/*           before returning. WARNING: If VI_TMO_INFINITE is passed in as the    */
/*           timeout this function WILL NOT return until the specified event is   */
/*           received, if the event is not received the function will not return  */
/*           and it will be necessary to terminate CVI in order to regain control.*/
/*================================================================================*/
ViStatus avopt66_waitOnVisaEvent (ViSession instrSession, ViEventType eventTypeIn, 
            ViUInt32 timeoutIn, ViPUInt16 STB)
{            
    ViStatus avopt66_status = VI_SUCCESS, tempStatus = VI_SUCCESS;
    ViEventType eventTypeOut = 0;
    ViEvent eventHandle = 0;
    
    /* For debug purposes we want to be able to see the status returned by        */
    /* viDisableEvent and viClose if one of the internal functions fails but do   */
    /* not want to return that value as that is not where the initial error       */
    /* occurs in the function, so we assign it to tempStatus.                     */
    
    if ((avopt66_status = viWaitOnEvent (instrSession, eventTypeIn, timeoutIn, 
                                        &eventTypeOut, &eventHandle)) < 0) {
        tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
        return avopt66_status;
    } 
    
    if (eventTypeIn == VI_EVENT_SERVICE_REQ)
        if ((avopt66_status = viReadSTB (instrSession, STB)) < 0) {
            tempStatus = viClose (eventHandle);
            tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
            return avopt66_status;
        }   
        
    if ((avopt66_status = viClose (eventHandle)) < 0) {             
        tempStatus = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE);
        return avopt66_status;                 
    }
    
    if ((avopt66_status = viDisableEvent (instrSession, eventTypeIn, VI_QUEUE)) < 0)
        return avopt66_status;                                                   

    return avopt66_status;                                    
}    

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the avopt66_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from avopt66_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus avopt66_initCleanUp (ViSession openRMSession,
                ViPSession openInstrSession, ViStatus currentStatus)
{
    avopt66_instrRange instrPtr;
    
    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL) 
            free (instrPtr);

    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;
    
    return currentStatus;
}

/*****************************************************************************/
/*----------- INSERT INSTRUMENT-DEPENDENT UTILITY ROUTINES HERE -------------*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the avopt66_reset operation and by the     */
/*           avopt66_init function if the reset option has not been           */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus avopt66_defaultInstrSetup (ViSession instrSession)
{
    ViStatus avopt66_status = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    avopt66_instrRange instrPtr;
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    if (avopt66_status = viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr))
        return avopt66_status;
    
    if (instrPtr == NULL) 
        instrPtr = malloc (sizeof (struct avopt66_statusDataRanges));

    instrPtr -> traceRange = 1001;
    instrPtr -> harmonics = 3;
    Fmt (instrPtr -> instrDriverRevision, "%s<%s", avopt66_REVISION);
    
    if ((avopt66_status = viSetAttribute (instrSession, VI_ATTR_USER_DATA, 
                            (ViUInt32)instrPtr)) < 0)
        return avopt66_status;                                       

    return avopt66_status;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
