#include <cvidef.h>

/*= R3267 Series - Modulation Spectrum Analyzes Include File ================*/

/*===========================================================================*/
/*  Please do not use global variables or arrays in the include file of      */
/*  instrument drivers that will be submitted for inclusion into the         */
/*  LabWindows Instrument Driver Library.                                    */
/*===========================================================================*/
     
#ifndef __avopt66_HEADER
#define __avopt66_HEADER

#include <vpptype.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

/*****************************************************************************/
/*= Instrument Driver Specific Error/Warning Codes ==========================*/
/*****************************************************************************/
#define VI_ERROR_INSTR_FILE_OPEN                (_VI_ERROR+0x3FFC0800L)
#define VI_ERROR_INSTR_FILE_WRITE               (_VI_ERROR+0x3FFC0801L)
#define VI_ERROR_INSTR_INTERPRETING_RESPONSE    (_VI_ERROR+0x3FFC0803L)
#define VI_ERROR_INSTR_PARAMETER9               (_VI_ERROR+0x3FFC0809L)
#define VI_ERROR_INSTR_PARAMETER10              (_VI_ERROR+0x3FFC080AL)
#define VI_ERROR_INSTR_PARAMETER11              (_VI_ERROR+0x3FFC080BL)
#define VI_ERROR_INSTR_PARAMETER12              (_VI_ERROR+0x3FFC080CL)
#define VI_ERROR_INSTR_PARAMETER13              (_VI_ERROR+0x3FFC080DL)
#define VI_ERROR_INSTR_PARAMETER14              (_VI_ERROR+0x3FFC080EL)
#define VI_ERROR_INSTR_PARAMETER15              (_VI_ERROR+0x3FFC080FL)
#define VI_ERROR_INSTR_PARAMETER16              (_VI_ERROR+0x3FFC0810L)
#define VI_ERROR_INSTR_PARAMETER17              (_VI_ERROR+0x3FFC0811L)
#define VI_ERROR_INSTR_PARAMETER18              (_VI_ERROR+0x3FFC0812L)        
#define VI_ERROR_INSTR_PARAMETER19              (_VI_ERROR+0x3FFC0813L)
#define VI_ERROR_INSTR_PARAMETER20              (_VI_ERROR+0x3FFC0814L)

/*****************************************************************************/
/*= Define Instrument Specific Error/Warning Codes Here =====================*/
/*****************************************************************************/
#define VI_WARNING_INSTR_OFFSET                           (0x3FFC0900L)
#define VI_ERROR_INSTR_OFFSET                   (_VI_ERROR+0x3FFC0900L)

#define avopt66_ERROR_INVALID_CONFIGURATION      (VI_ERROR_INSTR_OFFSET + 0xF0L)

/*****************************************************************************/
/*= GLOBAL USER-CALLABLE FUNCTION DECLARATIONS (Exportable Functions) =======*/
/*****************************************************************************/
ViStatus CVIFUNC avopt66_parameter_stdBT (ViSession instrumentHandle,
                                          ViInt16 CNAssignment, ViBoolean link,
                                          ViBoolean hoppingCatch,
                                          ViInt16 measMode, ViInt16 burstLength,
                                          ViInt16 searchLength,
                                          ViBoolean syncType,
                                          ViBoolean fillterMode, ViInt32 lap,
                                          ViReal64 offsetLevel,
                                          ViBoolean frequencyInput,
                                          ViBoolean input,
                                          ViBoolean basebandInput,
                                          ViBoolean IQInverse,
                                          ViBoolean contAutoLevelSet);

ViStatus CVIFUNC avopt66_number_setChannel (ViSession instrumentHandle,
                                            ViInt16 channelNo);

ViStatus CVIFUNC avopt66_ChEdt (ViSession instrumentHandle, ViBoolean settoSTD,
                                ViInt16 editChannel, ViInt16 startCHNum,
                                ViInt16 stopCHNum, ViReal64 baseFreqHz,
                                ViReal64 channelSpaceHz, ViInt16 CHOffset);

ViStatus CVIFUNC avopt66_ch_Enable (ViSession instrumentHandle,
                                    ViBoolean channel1, ViBoolean channel2,
                                    ViBoolean channel3);

ViStatus CVIFUNC avopt66_Yscale_setup (ViSession instrumentHandle,
                                       ViInt16 measMode, ViInt16 division);

ViStatus CVIFUNC avopt66_confTrgConTDP (ViSession instrumentHandle,
                                        ViInt16 triggerSource,
                                        ViBoolean trgSlope, ViInt16 triggerLevel,
                                        ViInt16 triggerPosition,
                                        ViReal64 delayTimes);

ViStatus CVIFUNC avopt66_windowSetTDP (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViBoolean window,
                                       ViReal64 winPosS, ViReal64 winWidS);

ViStatus CVIFUNC avopt66_template_SetTDP (ViSession instrumentHandle,
                                          ViBoolean template, ViReal64 shiftXS,
                                          ViReal64 shiftYdB);

ViStatus CVIFUNC avopt66_template_editTDP (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean templateSelect,
                                           ViReal64 templateTimes,
                                           ViReal64 templateLevel);

ViStatus CVIFUNC avopt66_parameter_setTDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean TMPLCoupleofPWR,
                                           ViReal64 templateLimit,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit);

ViStatus CVIFUNC avopt66_trigger_setOOR (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes);

ViStatus CVIFUNC avopt66_window_setOOR (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 onPositionS, ViReal64 onWidthS,
                                        ViReal64 offPositionS,
                                        ViReal64 offWidthS);

ViStatus CVIFUNC avopt66_param_setOOR (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViInt16 displayUnit, ViBoolean judgement,
                                       ViReal64 upperLimit);

ViStatus CVIFUNC avopt66_trigger_setTDS (ViSession instrumentHandle,
                                         ViInt16 triggerSource,
                                         ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes);

ViStatus CVIFUNC avopt66_table_editTDS (ViSession instrumentHandle,
                                        ViInt16 operation, ViInt16 tableNumber,
                                        ViBoolean freqType, ViReal64 carrierFreq,
                                        ViReal64 frequency, ViReal64 level);

ViStatus CVIFUNC avopt66_param_setTDS (ViSession instrumentHandle,
                                       ViBoolean settoSTD, ViInt16 detector,
                                       ViInt16 displayUnit, ViBoolean judgement,
                                       ViBoolean result, ViReal64 peakMKRYDelta,
                                       ViReal64 multiplier,
                                       ViBoolean preselector);

ViStatus CVIFUNC avopt66_gateTrg_setFDP (ViSession instrumentHandle,
                                         ViBoolean gateSource,
                                         ViBoolean settoSTD,
                                         ViBoolean measurementMode,
                                         ViReal64 gatePositions,
                                         ViReal64 gateWidths, ViInt16 trgSource,
                                         ViInt16 triggerPosition,
                                         ViReal64 delayTimes, ViBoolean trgSlope,
                                         ViInt16 triggerLevel,
                                         ViBoolean gatedSweep, ViInt16 detector);

ViStatus CVIFUNC avopt66_window_setFDP (ViSession instrumentHandle,
                                        ViBoolean settoSTD, ViBoolean window,
                                        ViReal64 positionHz, ViReal64 widthHz);

ViStatus CVIFUNC avopt66_parameter_setFDP (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViInt16 detector,
                                           ViInt16 displayUnit,
                                           ViBoolean gatedSweep,
                                           ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit);

ViStatus CVIFUNC avopt66_parameter_setOBW (ViSession instrumentHandle,
                                           ViBoolean settoSTD, ViReal64 OBW,
                                           ViInt16 detector, ViBoolean judgement,
                                           ViReal64 upperLimit,
                                           ViReal64 lowerLimit);

ViStatus CVIFUNC avopt66_parameter_setDT (ViSession instrumentHandle,
                                          ViBoolean settoSTD, ViInt16 detector,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViReal64 templateLimit,
                                          ViBoolean judgement,
                                          ViBoolean freqSetting, ViInt16 result,
                                          ViBoolean refPower,
                                          ViReal64 symbolRate1T,
                                          ViReal64 rolloffFactor);

ViStatus CVIFUNC avopt66_template_setDT (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz);

ViStatus CVIFUNC avopt66_template_editDT (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLeveldB);

ViStatus CVIFUNC avopt66_marker_editDT (ViSession instrumentHandle,
                                        ViInt16 operation, ViInt16 refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB);

ViStatus CVIFUNC avopt66_parameter_setDM (ViSession instrumentHandle,
                                          ViBoolean settoSTD,
                                          ViBoolean freqSetting,
                                          ViInt16 detector, ViInt16 result,
                                          ViBoolean refPower,
                                          ViInt16 displayUnit,
                                          ViBoolean TMPLCoupleofPWR,
                                          ViBoolean judgement,
                                          ViReal64 templateLimit,
                                          ViReal64 symbolRate1T,
                                          ViReal64 rolloffFactor);

ViStatus CVIFUNC avopt66_template_setDM (ViSession instrumentHandle,
                                         ViBoolean template, ViReal64 shiftXHz,
                                         ViReal64 shiftYdB, ViReal64 deltaXHz);

ViStatus CVIFUNC avopt66_template_editDM (ViSession instrumentHandle,
                                          ViInt16 operation,
                                          ViReal64 templateFreqHz,
                                          ViReal64 templateLeveldB);

ViStatus CVIFUNC avopt66_marker_editDM (ViSession instrumentHandle,
                                        ViInt16 operation,
                                        ViBoolean refMarkerType,
                                        ViReal64 refBandWidthHz,
                                        ViBoolean offsetMkrType,
                                        ViReal64 offsetFreqHz,
                                        ViReal64 bandWidthHz,
                                        ViReal64 limitPowerdB);

ViStatus CVIFUNC avopt66_parameter_setINSPR (ViSession instrumentHandle,
                                             ViBoolean settoSTD,
                                             ViBoolean freqSetting,
                                             ViInt16 detector,
                                             ViBoolean refPower,
                                             ViReal64 peakMkrYDelta,
                                             ViInt16 result, ViInt16 displayUnit,
                                             ViBoolean TMPLCoupleofPWR,
                                             ViReal64 templateLimit,
                                             ViBoolean judgement);

ViStatus CVIFUNC avopt66_template_setINSPR (ViSession instrumentHandle,
                                            ViBoolean template,
                                            ViReal64 shiftXHz, ViReal64 shiftYdB,
                                            ViReal64 deltaXHz);

ViStatus CVIFUNC avopt66_template_editINSPR (ViSession instrumentHandle,
                                             ViInt16 operation,
                                             ViReal64 templateFreqHz,
                                             ViReal64 templateLeveldB);

ViStatus CVIFUNC avopt66_marker_editINSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViBoolean refMarkerType,
                                           ViReal64 refBandWidthHz,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz,
                                           ViReal64 limitPowerdB);

ViStatus CVIFUNC avopt66_parameter_setOUTSPR (ViSession instrumentHandle,
                                              ViBoolean settoSTD,
                                              ViInt16 detector,
                                              ViReal64 peakMkrYDelta,
                                              ViInt16 displayUnit,
                                              ViBoolean judgement,
                                              ViBoolean preselector);

ViStatus CVIFUNC avopt66_table_editOUTSPR (ViSession instrumentHandle,
                                           ViInt16 operation,
                                           ViInt16 tableNumber,
                                           ViReal64 startFreqHz,
                                           ViReal64 stopFreqHz, ViReal64 RBW,
                                           ViReal64 VBW, ViReal64 sweepTime,
                                           ViReal64 limitLevel);

ViStatus CVIFUNC avopt66_powerTimeMeas_setGSM (ViSession instrumentHandle,
                                               ViBoolean powervsTimeMeasMode,
                                               ViInt16 YScale);

ViStatus CVIFUNC avopt66_parameter_setMOD (ViSession instrumentHandle,
                                           ViInt16 triggerSource,
                                           ViBoolean burstSwitch,
                                           ViReal64 searchLevel,
                                           ViBoolean trgSlope,
                                           ViInt16 triggerLevel,
                                           ViBoolean extTrgDelay,
                                           ViInt16 slotNumber,
                                           ViReal64 delayTimes,
                                           ViBoolean bitSequence);

ViStatus CVIFUNC avopt66_lockupTimGrpScl_setMOD (ViSession instrumentHandle,
                                                 ViReal64 XSpan, ViReal64 XStart,
                                                 ViInt16 YScale);

ViStatus CVIFUNC avopt66_lockupTimParam_setMOD (ViSession instrumentHandle,
                                                ViBoolean freqRange,
                                                ViReal64 analyzeLength,
                                                ViInt16 triggerSource,
                                                ViBoolean triggerSlop,
                                                ViInt16 triggerLevel,
                                                ViBoolean extTrgDelay,
                                                ViInt16 slotNumber,
                                                ViReal64 triggerDelay,
                                                ViReal64 limit);

ViStatus CVIFUNC avopt66_average_set (ViSession instrumentHandle,
                                      ViInt16 function, ViInt16 AVGTimes);


ViStatus CVIFUNC avopt66_execute_auto_lvl (ViSession instrumentHandle,
                                           ViInt16 function);

ViStatus CVIFUNC avopt66_act_DcCal (ViSession instrumentHandle);

ViStatus CVIFUNC avopt66_measStart_act (ViSession instrumentHandle,
                                        ViInt16 function);

ViStatus CVIFUNC avopt66_graphTyp_selFMD (ViSession instrumentHandle,
                                          ViInt16 graphType, ViInt16 startBit);

ViStatus CVIFUNC avopt66_check_measMode (ViSession instrumentHandle,
                                         ViInt16 *returnmode);

ViStatus CVIFUNC avopt66_data_TDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement);

ViStatus CVIFUNC avopt66_data_OnOffRatio (ViSession instrumentHandle,
                                          ViReal64 *ONLevel, ViReal64 *OFFLevel,
                                          ViReal64 *ONOFFRaito,
                                          ViInt16 *judgement);



ViStatus CVIFUNC avopt66_data_spuriTD (ViSession instrumentHandle,
                                       ViInt16 *tableSize,
                                       ViReal64 frequencyHz[], ViReal64 level[],
                                       ViInt16 judgement[]);

ViStatus CVIFUNC avopt66_data_FDP (ViSession instrumentHandle,
                                   ViReal64 *powerLeveldBm, ViInt16 *judgement);

ViStatus CVIFUNC avopt66_data_OBW (ViSession instrumentHandle, ViReal64 *OBWHz,
                                   ViReal64 *lowFreqHz, ViReal64 *hiFreqHz,
                                   ViInt16 *judgement);



ViStatus CVIFUNC avopt66_data_duetoTRN (ViSession instrumentHandle,
                                        ViInt16 *numberofACP,
                                        ViReal64 ACPDatadBc[],
                                        ViInt16 judgement[]);

ViStatus CVIFUNC avopt66_data_duetoMOD (ViSession instrumentHandle,
                                        ViInt16 *numberofACP, ViReal64 ACPdBc[],
                                        ViInt16 ACPJudge[]);

ViStatus CVIFUNC avopt66_data_INSPR (ViSession instrumentHandle,
                                     ViInt16 *numberofPoint,
                                     ViReal64 frequencyHz[], ViReal64 leveldBm[],
                                     ViInt16 judge[]);

ViStatus CVIFUNC avopt66_data_spuriOB (ViSession instrumentHandle,
                                       ViInt16 *numberofPoint,
                                       ViReal64 frequencyHz[],
                                       ViReal64 leveldBm[], ViInt16 judgement[]);

ViStatus CVIFUNC avopt66_data_TxPower (ViSession instrumentHandle,
                                       ViReal64 *txPowerdBm, ViReal64 *txPowerW,
                                       ViReal64 *peakTxPowerdBm,
                                       ViReal64 *posofpeakTxPower,
                                       ViReal64 *peakTxPowerW);

ViStatus CVIFUNC avopt66_data_LockupTime (ViSession instrumentHandle,
                                          ViReal64 *lockupTime);

ViStatus CVIFUNC avopt66_data_graphLkupTim (ViSession instrumentHandle,
                                            ViInt16 *XDataSize, ViReal64 XData[],
                                            ViInt16 *YDataSize, ViReal64 YData[]);

ViStatus CVIFUNC avopt66_dataFreqDevRandSeq (ViSession instrumentHandle,
                                             ViReal64 *frequencyError,
                                             ViReal64 *maxPeakDev,
                                             ViReal64 *posofMaxPeakDev,
                                             ViReal64 *minPeakDev,
                                             ViReal64 *posofMinPeakDev,
                                             ViReal64 *minMaxRaito,
                                             ViReal64 *txPowerdBm,
                                             ViReal64 *txPowerW,
                                             ViReal64 *peakTxPowerdBm,
                                             ViReal64 *posofPeakTxPower,
                                             ViReal64 *peakTxPowerW);

ViStatus CVIFUNC avopt66_data_fmSTD (ViSession instrumentHandle,
                                     ViReal64 *frequencyError,
                                     ViReal64 *maxPeakDev,
                                     ViReal64 *posofMaxPeakDev,
                                     ViReal64 *avgPeakDeviHz,
                                     ViReal64 *frequencyDriftHz,
                                     ViReal64 *maxDriftRaito,
                                     ViReal64 *txPowerdBm, ViReal64 *txPowerW,
                                     ViReal64 *peakTxPowerdBm,
                                     ViReal64 *posofPeakTxPower,
                                     ViReal64 *peakTxPowerW);


ViStatus CVIFUNC avopt66_dataGraphFreqDeviation (ViSession instrumentHandle,
                                                 ViInt16 startBit,
                                                 ViInt16 graphType,
                                                 ViInt16 *XDataSize,
                                                 ViReal64 XData[],
                                                 ViInt16 *YDataSize,
                                                 ViReal64 YData[],
                                                 ViInt16 *demodulatedDataSize,
                                                 ViInt32 demodulatedData[]);

ViStatus CVIFUNC avopt66_close (ViSession instrSession);

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif

/*****************************************************************************/
/*=== END INCLUDE FILE ======================================================*/
/*****************************************************************************/

#endif
